<?php

namespace app\commands;

use yii\console\Controller;
use yii;
use yii\helpers\ArrayHelper;
use app\components\purchases\ClientSiesa;

class FullfillmentClientController extends Controller {

    /**
     *
     */
    public function actionIndex() {
        Yii::$app->db->createCommand('TRUNCATE data_fulfillment_client')->execute();
        $this->nsFullfillmentClient();
    }

    /**
     * Metodo para extraer e insertar localmente la informacion de siesa 
     * para el calculo del nivel de servicio de clientes
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function nsFullfillmentClient() {
        $dateEnd = date('d-m-Y');
        $dateIn = date('d-m-Y', strtotime('-1 month'));
        $result = $this->_getData($dateIn, $dateEnd);

        if (count($result) > 0) {
            $arrayBlocks = array_chunk($result, 500);
            foreach ($arrayBlocks as $block) {
                $this->_insertData($block);
            }
        }
    }

    /**
     * Metodo para la insercion en bloque de la data
     * segun el proveedor
     * 
     * @param array $data
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _insertData($data) {
        try {
            if (empty($data) == false) {
                Yii::$app->db->createCommand()->batchInsert('data_fulfillment_client',
                        [
                            'year_order',
                            'year_invoice',
                            'year_default',
                            'month_order',
                            'month_invoice',
                            'month_default',
                            'operation_center',
                            'document_type_order',
                            'number_order',
                            'date_order',
                            'document_type_invoice',
                            'number_invoice',
                            'date_invoice',
                            'number_delivery',
                            'status_delivery',
                            'reason_delivery',
                            'date_delivery',
                            'nit',
                            'business_name',
                            'branch_code',
                            'branch_name',
                            'total_references',
                            'total_ref_inc',
                            'reference',
                            'description',
                            'quantity_order',
                            'unit_value_order',
                            'quantity_invoice',
                            'unit_value_invoice',
                            'total_unit_value_invoice',
                            'subtotal_order',
                            'subtotal_invoice',
                            'total_order', 'total_invoice',
                            'cant_pendiente',
                            'cantidad_devuelta',
                            'quantity_delivery'],
                        $data)->execute();
            }
        } catch (\Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r($exc->getMessage(), true) . '}');
        }
    }

    /**
     * Metodo para obtener la informacion de todos los movimientos
     * 
     * @param string $dateIn  Fecha Inicial fotmato(dd-mm-YYYY)
     * @param string $dateEnd Fecha Final fotmato(dd-mm-YYYY)
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getData($dateIn, $dateEnd) {
        $arrayResult = array();
        try {
            $sql = "
                SET NOCOUNT ON;
                SELECT
                    YEAR(f430_fecha_ts_cumplido) as year_order,
                    YEAR(f350_fecha) as year_invoice,
                    YEAR(f430_fecha_ts_cumplido) as year_default,
                    DATENAME(M, DATEADD(M, MONTH(f430_fecha_ts_cumplido), -1)) as month_order,
                    DATENAME(M, DATEADD(M, MONTH(f350_fecha), -1)) as month_invoice,
                    DATENAME(M, DATEADD(M, MONTH(f430_fecha_ts_cumplido), -1)) as month_default,
                    -- f430_fecha_ts_cumplido,
                    DRM.f460_id_co_docto AS operation_center, 
                    DPV.f430_id_tipo_docto AS document_type_order,	
                    DPV.f430_consec_docto AS number_order, 
                    CONVERT(DATE,MPV.f431_fecha_cumplido,103) AS date_order,
                    Docfac.f350_id_tipo_docto AS document_type_invoice,
                    Docfac.f350_consec_docto AS number_invoice, 
                    CONVERT(DATE,Docfac.f350_fecha,103)  AS date_invoice, 
                    -- PLN.f463_id_tipo_docto AS f463_id_tipo_docto, ------------------------------------
                    PLN.f463_consec_docto AS number_delivery,
                    -- CONVERT(DATE,PLN.f463_fecha,103)  AS f463_fecha,----------------------------------
                    PREL.f464_ind_estado AS status_delivery,
                    PREL.f464_notas AS reason_delivery,
                    CONVERT(DATE,PREL.f464_fecha_ts_cierre,103) as date_delivery,
                    LTRIM(RTRIM(T200.F200_NIT)) AS nit,
                    LTRIM(RTRIM(f200_razon_social)) AS business_name,
                    LTRIM(RTRIM(DFV.f461_id_sucursal_fact)) AS branch_code,
                    LTRIM(RTRIM(C201.f201_descripcion_sucursal)) AS branch_name,
                    -- DFV.f461_id_co_fact AS f461_id_co_fact, ----------------------------------
                    -- T120.f120_id AS f120_id,-------------------------------
                    COUNT(T120.f120_referencia) OVER() AS total_references,
                    SUM(CASE WHEN MPV.f431_cant1_pedida <> MPV.f431_cant1_facturada THEN 1 ELSE 0 END) AS total_ref_inc,
                    LTRIM(RTRIM(T120.f120_referencia)) AS reference,
                    LTRIM(RTRIM(T120.f120_descripcion)) AS description,
                    SUM(MPV.f431_cant1_pedida) AS quantity_order,
                    SUM(MPV.f431_precio_unitario_base) AS unit_value_order,
                    SUM(MPV.f431_cant1_facturada) AS quantity_invoice,
                    SUM(MPV.f431_vlr_bruto) AS unit_value_invoice,
                    SUM(MPV.f431_vlr_bruto) OVER() AS total_unit_value_invoice,
                    SUM(MPV.f431_cant1_pedida*MPV.f431_precio_unitario_base) as subtotal_order,
                    SUM(MPV.f431_precio_unitario_base*MPV.f431_cant1_facturada) as subtotal_invoice,
                    SUM(MPV.f431_cant1_pedida*MPV.f431_precio_unitario_base) OVER() AS total_order,
                    SUM(MPV.f431_precio_unitario_base*MPV.f431_cant1_facturada) OVER() AS total_invoice,
                    SUM(CASE WHEN DPV.f430_id_clase_docto = 508 THEN CASE WHEN f431_cant1_facturada > f431_cant1_pedida THEN 0
                    ELSE f431_cant1_pedida - f431_cant1_facturada END
                                     ELSE
                                             CASE
                                               WHEN f431_cant1_remisionada > f431_cant1_pedida THEN 0
                                               ELSE f431_cant1_pedida - f431_cant1_remisionada
                                             END
                             END) AS cant_pendiente,
                    SUM(ISNULL(NTA.Cantidad,0)) AS cantidad_devuelta,
                    SUM(MPV.f431_cant1_pedida-(CASE WHEN DPV.f430_id_clase_docto = 508 THEN CASE WHEN f431_cant1_facturada > f431_cant1_pedida THEN 0
                    ELSE f431_cant1_pedida - f431_cant1_facturada END
                        ELSE
                           CASE
                             WHEN f431_cant1_remisionada > f431_cant1_pedida THEN 0
                             ELSE f431_cant1_pedida - f431_cant1_remisionada
                           END
                        END)+ISNULL(NTA.Cantidad,0)) AS quantity_delivery
                FROM
                    t460_cm_docto_remision_venta DRM 

                left join t430_cm_pv_docto DPV
                On DRM.f460_rowid_pv_docto = DPV.f430_rowid

                LEFT JOIN t431_cm_pv_movto MPV
                ON DPV.f430_rowid = MPV.f431_rowid_pv_docto

                LEFT JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MPV.f431_rowid_item_ext

                LEFT JOIN  t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item

                left join t461_cm_docto_factura_venta DFV
                On DRM.f460_rowid_docto_factura = DFV.f461_rowid_docto

                LEFT JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = DFV.f461_rowid_docto

                LEFT JOIN t200_mm_terceros T200
                ON f461_rowid_tercero_fact = T200.f200_rowid

                left JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND DFV.f461_id_sucursal_fact = C201.F201_ID_SUCURSAL

                LEFT JOIN (
                    Select 
                        PREL.f464_rowid_docto_relacion,
                        PREL.f464_id_cia, 
                        PREL.f464_rowid_docto_planilla, 
                        PREL.f464_ind_estado,
                        PREL.f464_notas,
                        PREL.f464_fecha_ts_cierre,
                        ROW_NUMBER() OVER(PARTITION BY PREL.f464_rowid_docto_relacion ORDER BY PREL.f464_rowid_docto_planilla ASC) AS Rnum
                    From  t464_cm_docto_planilla_rel PREL
                    where PREL.f464_fecha_ts_creacion BETWEEN '$dateIn' AND '$dateEnd'
                    ) PREL 
                ON DOCFAC.f350_rowid = PREL.f464_rowid_docto_relacion
                and DOCFAC.f350_id_cia = PREL.f464_id_cia
                AND PREL.Rnum = 1

                inner join t463_cm_docto_planilla PLN
                On f464_rowid_docto_planilla = f463_rowid 

                LEFT JOIN (
                    Select 
                        docNc.f350_id_tipo_docto,
                        docNc.f350_consec_docto,
                        msa.f353_id_tipo_docto_cruce,
                        msa.f353_consec_docto_cruce,
                        msa.f353_id_co_cruce,
                        MFV.f470_rowid_item_ext,
                        sum((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END) ) AS Cantidad
                    from 
                        t470_cm_movto_invent MFV

                    inner join t350_co_docto_contable DOCNC
                    ON MFV.f470_rowid_docto_fact = DOCNC.f350_rowid

                    left join t352_co_docto_cruce t352  
                    ON MFV.f470_rowid_docto_fact = t352.f352_rowid_docto

                    inner join t353_co_saldo_abierto MSA
                    On t352.f352_rowid_sa = msa.f353_rowid
                    And t352.f352_id_cia = msa.f353_id_cia
                    and DOCNC.f350_id_co = msa.f353_id_co_cruce

                    where DOCNC.f350_fecha BETWEEN '$dateIn' AND '$dateEnd'

                    group by 
                        docNc.f350_id_tipo_docto,
                        docNc.f350_consec_docto,
                        msa.f353_id_tipo_docto_cruce,
                        msa.f353_consec_docto_cruce,
                        msa.f353_id_co_cruce,
                        MFV.f470_rowid_item_ext

                    ) NTA
                ON Docfac.f350_id_tipo_docto = NTA.f353_id_tipo_docto_cruce
                AND Docfac.f350_consec_docto = NTA.f353_consec_docto_cruce
                AND DOCFAC.f350_id_co = NTA.f353_id_co_cruce
                AND MPV.f431_rowid_item_ext = NTA.f470_rowid_item_ext

                WHERE f200_ind_cliente = 1 AND f200_ind_estado = 1 
                    AND Docfac.f350_fecha BETWEEN '$dateIn' AND '$dateEnd'

                GROUP BY 
                f430_fecha_ts_cumplido, 
                DRM.f460_id_co_docto, 
                DPV.f430_id_tipo_docto,	
                DPV.f430_consec_docto, 
                f431_fecha_cumplido,
                Docfac.f350_id_tipo_docto,
                Docfac.f350_consec_docto, 
                Docfac.f350_fecha, 
                PLN.f463_id_tipo_docto,
                PLN.f463_consec_docto,
                PLN.f463_fecha,
                PREL.f464_ind_estado,
                PREL.f464_notas,
                PREL.f464_fecha_ts_cierre,
                T200.F200_NIT,
                T200.f200_razon_social,
                DFV.f461_id_sucursal_fact,
                C201.f201_descripcion_sucursal,
                DFV.f461_id_co_fact,
                T120.f120_id,
                T120.f120_referencia,
                T120.f120_descripcion,
                f431_cant1_pedida,
                f431_precio_unitario_base,
                f431_cant1_facturada,
                MPV.f431_vlr_bruto
            ";
            $arrayResult = Yii::$app->dbsql->createCommand($sql)->queryAll();
        } catch (Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r($exc->getMessage(), true) . '}');
        }
        return $arrayResult;
    }

}
