<?php

/**
 * @link http://www.yiiframework.com/
 * @copyright Copyright (c) 2008 Yii Software LLC
 * @license http://www.yiiframework.com/license/
 */

namespace app\commands;

use yii\console\Controller;
use yii;
use yii\helpers\ArrayHelper;
use app\models\Pending;

/**
 * This command echoes the first argument that you have entered.
 *
 * This command is provided as an example for you to learn how to create console commands.
 *
 * @author Qiang Xue <qiang.xue@gmail.com>
 * @since 2.0
 */
class InsertPendingsController extends Controller {
    
    const DIFF_DAY = 1;
    const DIFF_YEAR = 1;
    
    /**
     * 
     */
    public function actionIndex() { 
		//\Yii::info($data, 'pending');
		\Yii::error('EXECUTE CRON #######', 'pending');
        $this->_getPendigns();
        $this->_deleteLastYear();
    }
    
    /**
     * Elimina los registros de los n anios 
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _deleteLastYear(){
        $date = date('Y-m-d', mktime(0, 0, 0, date('m'), date('d'), date('Y')-self::DIFF_YEAR));
        Pending::deleteAll("created <= '$date'");
    }
    
    /**
     * Metodo para consultar los pendientes y almacenarlos en la bd local
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getPendigns() {
        $result = $this->_getPendingsSiesa();
        $arryaInsert = [];
        if(count($result)>0){
            foreach ($result as $data) {
                $tempData['nit'] = $data['nit'];
                $tempData['business_name'] = $data['razon_social'];
                $tempData['branch_code'] = $data['id_sucursal'];
                $tempData['branch_name'] = $data['descripcion_sucursal'];
                $tempData['number_order'] = $data['pedido'];
                $tempData['reference'] = $data['referencia'];
                $tempData['description'] = $data['descripcion'];
				$tempData['operation_center'] = $data['operation_center'];
                $tempData['quantity'] = $data['cantidad_pendiente'];
                $tempData['cost'] = $data['valor_unitario'];
                $tempData['created'] = date('Y-m-d', mktime(0, 0, 0, date('m'), date('d')-self::DIFF_DAY, date('Y')));
                $arryaData[] = $tempData;
            }
        }
        $column = [
            'nit',
            'business_name',
            'branch_code',
            'branch_name',
            'number_order',
            'reference',
            'description',
			'operation_center',
            'quantity',
            'cost',
            'created'
        ];
        try{
            $command = Yii::$app->db->createCommand()
                ->batchInsert('pending', $column, $arryaData)->execute();
        } catch (\Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}');
        }
    }
    
    /**
     * Retorna la informacion de SIESA para la consulta de pendientes
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private static function _getPendingsSiesa() {
        $date = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d')-self::DIFF_DAY, date('Y')));
        $arrayResultQuery = [];
        //$dateEnd = '29-10-2019'; // dato de pruebas ----------------------------------------------
        try {
            $query = "
                SET NOCOUNT ON;			

                SELECT 
                    f430_id_tipo_docto+'-'+CAST(f430_consec_docto as varchar) as pedido,
                    f431_fecha_cumplido as fecha,
                    T200.F200_NIT AS nit,
                    T200.f200_razon_social AS razon_social,
                    f430_id_sucursal_fact AS id_sucursal,
                    C201.f201_descripcion_sucursal AS descripcion_sucursal,
                    f430_id_co as operation_center,
                    T120.f120_referencia as referencia,
                    T120.f120_descripcion as descripcion,
                    (CASE WHEN f431_cant1_facturada > 0 THEN f431_cant1_pedida - f431_cant1_facturada ELSE f431_cant1_pedida - f431_cant1_remisionada END) AS cantidad_pendiente,
                    (CASE WHEN f431_cant1_facturada > 0 THEN f431_cant1_pedida - f431_cant1_facturada ELSE f431_cant1_pedida - f431_cant1_remisionada END)*(f431_vlr_bruto/f431_cant1_pedida) AS valor_unitario
                FROM
                T430_CM_PV_DOCTO PV 

                LEFT JOIN T431_CM_PV_MOVTO MPV
                ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                LEFT JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MPV.f431_rowid_item_ext

                left JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item
                
                left JOIN t200_mm_terceros T200
                ON f430_rowid_tercero_fact = T200.f200_rowid
                
                left JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND f430_id_sucursal_fact = C201.F201_ID_SUCURSAL

                Where
                    f431_fecha_cumplido = '$date'
                    AND PV.f430_ind_estado IN (3,4)
                 GROUP BY 
                 f430_id_tipo_docto, f430_consec_docto, f431_fecha_cumplido, f120_referencia,
                 f120_descripcion, f431_cant1_pedida, f431_cant1_facturada, f431_cant1_remisionada,
                 f431_vlr_bruto, F200_NIT, f200_razon_social, f430_id_sucursal_fact, 
				 f201_descripcion_sucursal, f430_id_co
                HAVING (CASE WHEN f431_cant1_facturada > 0 THEN f431_cant1_pedida - f431_cant1_facturada ELSE f431_cant1_pedida - f431_cant1_remisionada END) > 0
                ";
				\Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($query, true) . '}');
            $arrayResultQuery = Yii::$app->dbsql->createCommand($query)->queryAll();              
        } catch (\Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}');
        }
        return $arrayResultQuery;
    }
       
}
