<?php
namespace app\components;

use Yii;
use yii\base\Component;

/**
 * HelperReport.php
 *
 * PHP Version 5.3.0
 * 
 * @category  Helper
 * @package   components
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2019 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version   SVN: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */

/**
 * HelperReport.php
 *
 * // SUS COMENTARIOS AQUI
 *
 * @category  Helper
 * @package   components
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2019 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version   Release: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */
class HelperReport extends Component {
    
    private $params;
    
    /**
     * Constructor de la clase
     * 
     * @return void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2019 INGENEO S.A.S.     
     * @access public     
     */
    public function __construct($params = null)
    {
        $this->params = $params;
    }

    /**
     * Funcion desctructora 
     * 
     * @return void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2019 INGENEO S.A.S. 
     * @access public     
     */
    public function __destruct()
    {
        
    }
    
    /**
     * Retorna el listado a partir de un array y un string dado
     * 
     * @param string $string String separado por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getTempList($string, $arrayParams) {
        return $this->_getDataFromList($string, $arrayParams);
    }

    /**
     * Retorna el array de los parametros que iran el en select de la consulta 
     * de ventas
     * 
     * @param string $params Parametros separados por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getListParams($paramsSelect) {
        return $this->_getDataFromList($this->params, $paramsSelect);
    }
    

    /**
     * Funcion que busca los datos de un string en un array
     * 
     * @param string $string Datos separados por comas    
     * @param array  $list   Array donde se buscaran los datos
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getDataFromList($string, $list) {
        $array = explode(',', $string);
        $arrayResul = [];
        if (count($array) > 0) {
            foreach ($array as $value) {
                $trimValue = trim($value);
                if (isset($list[$trimValue])) {
                    $arrayResul[$trimValue] = $list[$trimValue];
                }
            }
        }
        return $arrayResul;
    }
    
    /**
     * Funcion recursiva para extraer los datos
     * 
     * @param array $categories
     * @param array $datos
     * @param array $resultsDataY
     * @param array $tempEjeY
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function recursiveWalk(&$categories, $datos, &$resultsDataY, $tempEjeY){
        $objAux = [];
        $i = 0;
        if(is_array($datos) && count($datos)>0 ){
            foreach($datos as $key => $dato){
                if(isset($dato[$tempEjeY[0]])){
                    $categories[] = $key;
                    foreach ($tempEjeY as $value) {                  
                        $resultsDataY[$value][][$value] = $dato[$value];
                    }
                } else {
                    $objAuxJson = [];
                    $this->recursiveWalk($objAuxJson, $dato, $resultsDataY, $tempEjeY);
                    $objAux['name'] = $key;
                    $objAux['categories'] = $objAuxJson;
                    $categories[] = $objAux;
                }
                if(++$i == 5) {break;}
            }
        }        
    }

}

?>
