<?php

namespace app\controllers;

use Yii;
use app\models\Cmpuser;
use app\models\CmpuserSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\components\RijndaelHex;

/**
 * CmpuserController implements the CRUD actions for Cmpuser model.
 */
class CmpuserController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Cmpuser models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new CmpuserSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Cmpuser model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Cmpuser model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Cmpuser();
        $model->scenario = "crearuUs"; 
        
        if ($model->load(Yii::$app->request->post())) {              
            if ($model->validate()) { 
                $codeEncrypt = \Yii::$app->params['codeEncrypt'];
                $encrypter = new RijndaelHex($codeEncrypt);
                $encryptedPass = $encrypter->linencrypt($model->usr_password);
                $model->usr_password = $encryptedPass;
                $model->usr_password_repeat = $encryptedPass;
                if ($model->save()) {
                    return $this->redirect(['view', 'id' => $model->usr_user_ID]);
                }else{
                    return $this->render('create', ['model' => $model]);
                }
            } else {
                return $this->render('create', ['model' => $model]);
            }
        } else {
            return $this->render('create', ['model' => $model]);
        }
    }

    /**
     * Updates an existing Cmpuser model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);            

        $changeLogin = false;
        $beforePassword = $model->usr_password;
        $OldLogin = $model->usr_password;
        $model->usr_password = "";                        
        
        $model->scenario = "updateUs";        

        if ($model->load(Yii::$app->request->post())) {
            //VERIFICO SI SE CAMBIO EL LOGIN
            if ($OldLogin != $model->usr_username) {
                $changeLogin = true;
            }
            $password = '';
            $password_repeat = '';
            if (isset($_POST['Cmpuser']['usr_password']) && isset($_POST['Cmpuser']['usr_password_repeat'])) {                
                $password = $_POST['Cmpuser']['usr_password'];
                $password_repeat = $_POST['Cmpuser']['usr_password_repeat'];
            }
            $password = trim($password);
            $password_repeat = trim($password_repeat);
            if (empty($password)) {
                $model->usr_password = $model->usr_password_repeat = $beforePassword;
            }

            if ($model->validate()) {
                $codeEncrypt = \Yii::$app->params['codeEncrypt'];
                $encrypter = new RijndaelHex($codeEncrypt);
                $encryptedPass = $encrypter->linencrypt($password);
                $model->usr_password = $encryptedPass;
                $model->usr_password_repeat = $encryptedPass;
                if ($model->save()) {
                    if ($changeLogin) {
                        Yii::$app->user->logout();
                        return $this->redirect(['site/login']);
                    }
                    return $this->redirect(['update', 'id' => $model->usr_user_ID]);
                }
            } else {
                $model->usr_password = $password;
                $model->usr_password_repeat = $password_repeat;
                return $this->render('create', ['model' => $model]);
            }                        

            return $this->redirect(['update', 'id' => $model->usr_user_ID]);
        } else {
            return $this->render('update', ['model' => $model]);
        }
    }

    /**
     * Deletes an existing Cmpuser model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Cmpuser model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Cmpuser the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Cmpuser::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }
}
