<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;
use app\components\purchases\ClientSiesa;

/**
 * This is the model class for table "differences".
 *
 * @property integer $id
 * @property string $nit
 * @property string $business_name
 * @property string $branch_code
 * @property string $branch_name
 * @property string $reference
 * @property string $description
 * @property double $quantity
 * @property string $reason
 * @property string $crated
 * @property string $created_by
 * @property string $updated
 * @property string $updated_by
 */
class Differences extends ActiveRecord
{
    const VALIDATEDIFF = 'validateDiff';
    const SAVEDIFF = 'saveDiff';
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'differences';
    }
    
    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'updated'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['updated'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'updated_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['updated_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                Yii::$app->user->identity->username : '',
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['nit', 'business_name', 'branch_code', 'branch_name'], 'required', 
                'on'=> self::VALIDATEDIFF],
            [['nit', 'business_name', 'branch_code', 'branch_name', 'reference', 
                'description', 'quantity', 'reason'], 'required', 'on'=> self::SAVEDIFF],
            [['nit'], 'integer'],
            [['quantity'], 'number'],
            [['crated', 'updated'], 'safe'],
            [['business_name', 'branch_name', 'description', 'reason'], 
                'string', 'max' => 250],
            [['branch_code', 'reference'], 'string', 'max' => 45],
            [['created_by', 'updated_by'], 'string', 'max' => 150],
        ];
    }
    
    public function scenarios()
    {
        $scenarios = parent::scenarios();
        $scenarios[self::VALIDATEDIFF] = ['nit', 'business_name', 'branch_code', 
            'branch_name'];
        $scenarios[self::SAVEDIFF] = ['nit', 'business_name', 'branch_code', 
            'branch_name', 'reference','description', 'quantity', 'reason'];

        return $scenarios;
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'nit' => Yii::t('app', 'Nit'),
            'business_name' => Yii::t('app', 'Business Name'),
            'branch_code' => Yii::t('app', 'Branch Code'),
            'branch_name' => Yii::t('app', 'Branch Name'),
            'reference' => Yii::t('app', 'Reference'),
            'description' => Yii::t('app', 'Description'),
            'quantity' => Yii::t('app', 'Quantity'),
            'reason' => Yii::t('app', 'Reason'),
            'crated' => Yii::t('app', 'Crated'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated' => Yii::t('app', 'Updated'),
            'updated_by' => Yii::t('app', 'Updated By'),
        ];
    }
    
    /**
     * Retorna los productos asociados a un nit y sucursal
     * 
     * @param string $nit         Nit de la sucursal
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getItems(){
        $siesa = new ClientSiesa;
        $dateDiff = ($this->dateDiff($this->start_date, $this->end_date)+1);
        return $siesa->getItems($this->nit, $this->branch_code, $this->cellar, 
                $this->start_date, $this->end_date, $dateDiff, $this->projection_day);
    }
}
