<?php

namespace app\models;

use Yii;
use yii\helpers\ArrayHelper;
use app\components\ReportSiesa;
use yii\data\ArrayDataProvider;
use app\components\HelperReport;
use kartik\grid\GridView;
use kartik\select2\Select2;
use app\models\BeforeModel;

/**
 * This is the model class for table "report_fulfillment_provision".
 *
 * @property int $id
 * @property string $name
 * @property string $paramsAdd
 * @property string $paramsDefault
 * @property string $filter
 * @property string $ejex
 * @property string $ejey
 * @property string $type
 * @property string $created
 * @property string $created_by
 * @property string $updated
 * @property string $updated_by
 * @property int $user_id
 *
 * @property Cmpuser $user
 */
class Fulfillmentprovision extends BeforeModel
{
    public $type;
    public $date;
    public $operation_center;
    public $value_fulfillment_unit;
    public $value_fulfillment_references;
    public $value_fulfillment_order;
    public $value_fulfillment_money;
    public $fulfillment_unit;
    public $fulfillment_references;
    public $fulfillment_order;
    public $fulfillment_money;
    public $fulfillment_total;
    public $classification;
    public $month;
    public $year;
    
    const PENDINGS = 'PENDIENTES';
    const BILLS = 'FACTURAS';
    const ORDERS = 'PEDIDOS';
    const ORDERS_PENDINGS = 'PEDIDOS PENDIENTES';
    const ORDERS_FULL = 'PEDIDOS COMPLETOS';
    const ORDERS_TOTAL = 'TOTAL PEDIDOS';
    
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'report_fulfillment_provision';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'user_id'], 'required', 'on' => ['create', 'update']],
            [['user_id'], 'required', 'on' => ['preview']],
            [['paramsAdd', 'paramsDefault', 'filter', 'ejex', 'ejey'], 'string'],
            [['created', 'updated'], 'safe'],
            [['user_id'], 'integer'],
            [['name', 'created_by', 'updated_by'], 'string', 'max' => 150],
            [['type'], 'string', 'max' => 50],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => Cmpuser::className(), 'targetAttribute' => ['user_id' => 'usr_user_ID']],
            [['type','date','value_fulfillment_unit',
                'value_fulfillment_references',
                'value_fulfillment_order','value_fulfillment_money',
                'fulfillment_unit','fulfillment_references',
                'fulfillment_order','fulfillment_money','fulfillment_total',
                'classification','month','year', 'operation_center'], 'safe'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'name' => Yii::t('app', 'Name'),
            'paramsAdd' => Yii::t('app', 'Params Add'),
            'paramsDefault' => Yii::t('app', 'Params Default'),
            'filter' => Yii::t('app', 'Filter'),
            'ejex' => Yii::t('app', 'Ejex'),
            'ejey' => Yii::t('app', 'Ejey'),
            'type' => Yii::t('app', 'Type'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated' => Yii::t('app', 'Updated'),
            'updated_by' => Yii::t('app', 'Updated By'),
            'user_id' => Yii::t('app', 'User ID'),
        ];
    }
    
    public function scenarios()
    {
        $scenarios = parent::scenarios();        
        $scenarios['create'] = ['name', 'paramsDefault', 'user_id',
                'type','date','value_fulfillment_unit',
                'value_fulfillment_references', 'operation_center',
                'value_fulfillment_order','value_fulfillment_money',
                'fulfillment_unit','fulfillment_references',
                'fulfillment_order','fulfillment_money',
                'classification','month','year', 'fulfillment_total'];        
        $scenarios['update'] = ['name', 'paramsDefault', 'user_id',
                'type','date','value_fulfillment_unit',
                'value_fulfillment_references', 'operation_center',
                'value_fulfillment_order','value_fulfillment_money',
                'fulfillment_unit','fulfillment_references',
                'fulfillment_order','fulfillment_money',
                'classification','month','year','fulfillment_total'];        
        $scenarios['preview'] = ['user_id',
                'type','date','value_fulfillment_unit',
                'value_fulfillment_references', 'operation_center',
                'value_fulfillment_order','value_fulfillment_money',
                'fulfillment_unit','fulfillment_references',
                'fulfillment_order','fulfillment_money',
                'classification','month','year', 'fulfillment_total'];        
        return $scenarios;
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(Cmpuser::className(), ['usr_user_ID' => 'user_id']);
    }
    
    /**
     * Listado de los parametros por defecto sin espacios
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsDefault() {
        $list = 'type,date,year,month,fulfillment_unit,fulfillment_references,'
                . 'fulfillment_order,fulfillment_money';
        return str_replace(' ', '', $list);
    }

    /**
     * Listado de los parametros adicionales sin espacios
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsAdd() {
        $list = 'value_fulfillment_unit,value_fulfillment_references,'
                . 'value_fulfillment_order,value_fulfillment_money,'
                . 'operation_center,fulfillment_total';
        return str_replace(' ', '', $list);
    }
    
    
    /**
     * Lisrados de parametros configurables en el reporte
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParams() {
        return [
            'type' => ['content' => Yii::t('app', 'Type')],
            'date' => ['content' => Yii::t('app', 'Date')],
            'operation_center' => ['content' => Yii::t('app', 'Operation Center')],
            'value_fulfillment_references' => ['content' => Yii::t('app', 'Value Fulfillment References')],
            'value_fulfillment_unit' => ['content' => Yii::t('app', 'Value Fulfillment Unit')],
            'value_fulfillment_order' => ['content' => Yii::t('app', 'Value Fulfillment Order')],
            'value_fulfillment_money' => ['content' => Yii::t('app', 'Value Fulfillment Money')],
            'fulfillment_unit' => ['content' => Yii::t('app', 'Fulfillment Unit')],
            'fulfillment_references' => ['content' => Yii::t('app', 'Fulfillment References')],
            'fulfillment_order' => ['content' => Yii::t('app', 'Fulfillment Order')],
            'fulfillment_money' => ['content' => Yii::t('app', 'Fulfillment Money')],
            'fulfillment_total' => ['content' => Yii::t('app', 'Fulfillment Total')],
            'classification' => ['content' => Yii::t('app', 'Classification')],
            'month' => ['content' => Yii::t('app', 'Month')],
            'year' => ['content' => Yii::t('app', 'Year')],
        ];
    }
    
    /**
     * Metodo que retorna el listado de las columnas para el grid
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsGrid() {
        # Traemos todos los meses para el filtro -------------------------------
        $months = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
        $total = $count = 0;
        return [
            'type' => [
                'attribute' => 'type',
                'header' => Yii::t('app', 'Type'),
            ],   
            'operation_center' => [
                'attribute' => 'operation_center',
                'header' => Yii::t('app', 'Operation Center'),
            ],
            'date' => [
                'attribute' => 'date',
                'header' => Yii::t('app', 'Date'),
                'filterType' => GridView::FILTER_DATE_RANGE,
                'filterWidgetOptions' => [
                    'options' => ['placeholder' => 'Rango Fechas', 'readonly'=>true],
                    'pluginOptions' => [
                        'allowClear' => true,
                        'locale'=>[                            
                            'format' => 'DD-MM-YYYY',
                            'separator'=>' - ',
                        ],                        
                    ],
                ],                
                'width' => '200px',
                'value' => function($data){
                    return date("Y-m-d", strtotime($data['date']));
                }
            ],
            'value_fulfillment_unit' => [
                'attribute' => 'value_fulfillment_unit',
                'header' => Yii::t('app', 'Value Fulfillment Unit'),
                'value' => function($data){
                    if($data['value_fulfillment_unit'] == 0){
                        return '';
                    }else{
                        return number_format($data['value_fulfillment_unit'], 2);
                    }
                }
            ],
            'value_fulfillment_references' => [
                'attribute' => 'value_fulfillment_references',
                'header' => Yii::t('app', 'Value Fulfillment References'),
                'value' => function($data){
                    if($data['value_fulfillment_references'] == 0){
                        return '';
                    }else{
                        return number_format($data['value_fulfillment_references'], 2);
                    }
                }
            ],
            'value_fulfillment_order' => [
                'attribute' => 'value_fulfillment_order',
                'header' => Yii::t('app', 'Value Fulfillment Order'),
                'value' => function($data){
                    if($data['value_fulfillment_order'] == 0){
                        return '';
                    }else{
                        return number_format($data['value_fulfillment_order'], 2);
                    }
                }
            ],
            'value_fulfillment_money' => [
                'attribute' => 'value_fulfillment_money',
                'header' => Yii::t('app', 'Value Fulfillment Money'),
                'value' => function($data){
                    if($data['value_fulfillment_money'] == 0){
                        return '';
                    }else{
                        return '$ '.number_format($data['value_fulfillment_money'], 2);
                    }
                }
            ],
            'fulfillment_unit' => [
                'attribute' => 'fulfillment_unit',
                'header' => Yii::t('app', 'Fulfillment Unit'),
                'filter' => false,
                'value' => function($data){
                    if($data['fulfillment_unit'] == 0){
                        return '';
                    }else{
                        return '% '.number_format($data['fulfillment_unit'], 2);
                    }
                }
            ],
            'fulfillment_references' => [
                'attribute' => 'fulfillment_references',
                'header' => Yii::t('app', 'Fulfillment References'),
                'filter' => false,
                'value' => function($data){
                    if($data['fulfillment_references'] == 0){
                        return '';
                    }else{
                        return '% '.number_format($data['fulfillment_references'], 2);
                    }
                }
            ],
            'fulfillment_order' => [
                'attribute' => 'fulfillment_order',
                'header' => Yii::t('app', 'Fulfillment Order'),
                'filter' => false,
                'value' => function($data){
                    if($data['fulfillment_order'] == 0){
                        return '';
                    }else{
                        return '% '.number_format($data['fulfillment_order'], 2);
                    }
                },
            ],
            'fulfillment_money' => [
                'attribute' => 'fulfillment_money',
                'header' => Yii::t('app', 'Fulfillment Money'),
                'filter' => false,
                'value' => function($data){
                    if($data['fulfillment_money'] == 0){
                        return '';
                    }else{
                        return '% '.number_format($data['fulfillment_money'], 2);
                    }
                },
            ],
            'fulfillment_total' => [
                'attribute' => 'fulfillment_total',
                'header' => Yii::t('app', 'Fulfillment Total'),
                'filter' => false,
                'pageSummary'=>false,
                'format'=>['decimal', 2],
                'value' => function($data, $key, $index, $widget) use (&$total, &$count){
                    $count = $count + 1;
                    $value = 0;
                    if(isset($data['type'])){
                        switch ($data['type']) {
                            case Fulfillmentprovision::BILLS:
                                $value = $data['fulfillment_total'];
                                break;
                            case Fulfillmentprovision::ORDERS_FULL:
                                $value = $data['fulfillment_total'];
                                break;
                            default:
                                $value = 0;
                                break;
                        }
                        $total = $total + $value;
                        $widget->footer = number_format(($total),2);
                        return $value;
                    }else{
                        $total = $total + $data['fulfillment_total'];
                        $widget->footer = number_format(($total / $count),2);
                        return number_format($data['fulfillment_total'], 2);
                    }
                },
            ],
            'month' =>
            [                
                'attribute' => 'month',
                'header' => Yii::t('app', 'Month'),
                'width' => '150px',
                'filter'=>$months, 
                'filterType'=>  GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'theme' => Select2::THEME_BOOTSTRAP,
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
                'value' => function($model) use($months){            
                    return isset($months[$model['month']-1]) ? $months[$model['month']-1] : null;
                },                                
                'group'=>true, 
                'groupedRow'=>false,
                'subGroupOf'=>'year',
            ],
            'year' => [                
                'attribute' => 'year',
                'header' => Yii::t('app', 'Year'),
                'value' => 'year',                                
                'group'=>true, 
                'groupedRow'=>false, 
                'filter'=>array_combine(range(date('Y'), date('Y')-5),range(date('Y'), date('Y')-5))
            ],
        ];
    }
    
    /**
     * Retorna los parametrso que van en el select del B2BiSiesa
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsSelect($params) {  
        $fulfillment_unit = "IFNULL((SUM( value_fulfillment_unit) / IFNULL(sum(total_unit),0))*100,0)";
        $fulfillment_unit = "(CASE  
                    WHEN ($fulfillment_unit) > 100  THEN 100
                    WHEN ($fulfillment_unit) < 0 THEN 0
                    ELSE ($fulfillment_unit) END)";
        $fulfillment_references = "IFNULL((CAST(SUM( value_fulfillment_references) AS DECIMAL(15,2)) / CAST(IFNULL(sum(total_references),0) AS DECIMAL(15,2)))*100,0)";
        $fulfillment_references = "(CASE  
                    WHEN $fulfillment_references > 100 THEN 100
                    WHEN $fulfillment_references < 0 THEN 0
                    ELSE $fulfillment_references END)";
        $fulfillment_order = "IFNULL((CAST(SUM( value_fulfillment_order) AS DECIMAL(15,2)) / CAST(IFNULL(sum(total_order),0) AS DECIMAL(15,2)))*100,0)";
        $fulfillment_order = "(CASE  
                    WHEN $fulfillment_order > 100  THEN 100
                    WHEN $fulfillment_order < 0 THEN 0
                    ELSE $fulfillment_order END)";
        $fulfillment_money = "IFNULL((SUM( value_fulfillment_money) / IFNULL(sum(total_money),0))*100,0)";
        $fulfillment_money = "(CASE  
                    WHEN $fulfillment_money > 100  THEN 100
                    WHEN $fulfillment_money < 0 THEN 0
                    ELSE $fulfillment_money END)";
        $fulfillment_total = '(('.$fulfillment_unit.'+'.$fulfillment_money.'+'.$fulfillment_references.')*0.3)'
                    .'+('.$fulfillment_order.'*0.1)';
        return [
            'type' => ' type AS type',
            'date' => ' date AS date',
            'value_fulfillment_unit' => 'SUM( value_fulfillment_unit) AS value_fulfillment_unit',
            'value_fulfillment_references' => 'sum( value_fulfillment_references) as value_fulfillment_references',
            'value_fulfillment_order' => 'sum( value_fulfillment_order) as value_fulfillment_order',
            'value_fulfillment_money' => 'SUM( value_fulfillment_money) AS value_fulfillment_money',
            'fulfillment_unit' => "$fulfillment_unit AS fulfillment_unit",
            'fulfillment_references' => "$fulfillment_references AS fulfillment_references",
            'fulfillment_order' => "$fulfillment_order as fulfillment_order",
            'fulfillment_money' => "$fulfillment_money AS fulfillment_money",
            'fulfillment_total' => "$fulfillment_total AS fulfillment_total",
            'operation_center' => ' operation_center as operation_center',
            //'classification' => '',
            'year' => 'YEAR(date) as year',
            'month' => "MONTH(date) as month"
        ];
    }
    
    /**
     * Retorna los parametrso que van en el select del B2BiSiesa
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsSelectGraph($params) {                        
        $fulfillment_unit = "IFNULL((SUM( value_fulfillment_unit) / IFNULL(sum(total_unit),0))*100,0)";
        $fulfillment_unit = "(CASE  
                    WHEN ($fulfillment_unit) > 100  THEN 100
                    WHEN ($fulfillment_unit) < 0 THEN 0
                    ELSE ($fulfillment_unit) END)";
        $fulfillment_references = "IFNULL((CAST(SUM(value_fulfillment_references) AS DECIMAL(15,2)) / CAST(IFNULL(sum(total_references),0) AS DECIMAL(15,2)))*100,0)";
        $fulfillment_references = "(CASE  
                    WHEN $fulfillment_references > 100 THEN 100
                    WHEN $fulfillment_references < 0 THEN 0
                    ELSE $fulfillment_references END)";
        $fulfillment_order = "IFNULL((CAST(SUM(value_fulfillment_order) AS DECIMAL(15,2)) / CAST(IFNULL(sum(total_order),0) AS DECIMAL(15,2)))*100,0)";
        $fulfillment_order = "(CASE  
                    WHEN $fulfillment_order > 100  THEN 100
                    WHEN $fulfillment_order < 0 THEN 0
                    ELSE $fulfillment_order END)";
        $fulfillment_money = "IFNULL((SUM(value_fulfillment_money) / IFNULL(sum(total_money),0))*100,0)";
        $fulfillment_money = "(CASE  
                    WHEN $fulfillment_money > 100  THEN 100
                    WHEN $fulfillment_money < 0 THEN 0
                    ELSE $fulfillment_money END)";
        $fulfillment_total = '(('.$fulfillment_unit.'+'.$fulfillment_money.'+'.$fulfillment_references.')*0.3)'
                    .'+('.$fulfillment_order.'*0.1)';
        return [
            'type' => ' type AS type',
            'date' => ' date AS date',
            'value_fulfillment_unit' => 'SUM( value_fulfillment_unit) AS value_fulfillment_unit',
            'value_fulfillment_references' => 'sum( value_fulfillment_references) as value_fulfillment_references',
            'value_fulfillment_order' => 'sum( value_fulfillment_order) as value_fulfillment_order',
            'value_fulfillment_money' => 'SUM( value_fulfillment_money) AS value_fulfillment_money',
            'fulfillment_unit' => "$fulfillment_unit AS fulfillment_unit",
            'fulfillment_references' => "$fulfillment_references AS fulfillment_references",
            'fulfillment_order' => "$fulfillment_order as fulfillment_order",
            'fulfillment_money' => "$fulfillment_money AS fulfillment_money",
            'fulfillment_total' => "$fulfillment_total AS fulfillment_total",
            'operation_center' => ' operation_center as operation_center',
            //'classification' => '',
            'year' => 'YEAR( date) as year',
            'month' => "MONTHNAME(date) as month"
        ];
    }
    
    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsGroupBy($params) {  
        return [
            'type' => ' type, idtype',
            'date' => ' date',
            'operation_center' => ' operation_center',
//            'value_fulfillment_unit' => '',
//            'value_fulfillment_references' => '',
//            'value_fulfillment_order' => '',
//            'value_fulfillment_money' => '',
//            'fulfillment_unit' => '',
//            'fulfillment_references' => '',
//            'fulfillment_order' => '',
//            'fulfillment_money' => '',
//            'classification' => '',
            'year' => 'YEAR(date)',
            'month' => 'MONTH(date)',
        ];
    }
    
    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsGroupByGraph($params) {   
        return [
            'type' => ' type, idtype',
            'date' => ' date',
            'operation_center' => ' operation_center',
//            'value_fulfillment_unit' => '',
//            'value_fulfillment_references' => '',
//            'value_fulfillment_order' => '',
//            'value_fulfillment_money' => '',
//            'fulfillment_unit' => '',
//            'fulfillment_references' => '',
//            'fulfillment_order' => '',
//            'fulfillment_money' => '',
//            'classification' => '',
            'year' => 'YEAR(date)',
            'month' => 'MONTH(date)',
        ];
    }
    
    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsOrderBy($params) {
        return [
            'type' => ' idtype',
            'date' => ' date',
            'operation_center' => ' operation_center',
//            'value_fulfillment_unit' => '',
//            'value_fulfillment_references' => '',
//            'value_fulfillment_order' => '',
//            'value_fulfillment_money' => '',
//            'fulfillment_unit' => '',
//            'fulfillment_references' => '',
//            'fulfillment_order' => '',
//            'fulfillment_money' => '',
//            'classification' => '',
            'year' => 'YEAR(date)',
            'month' => 'MONTH(date)',
        ];
    }
    
    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsOrderByGraph($params) {
        return [
            'type' => ' idtype',
            'date' => ' date',
            'operation_center' => ' operation_center',
//            'value_fulfillment_unit' => '',
//            'value_fulfillment_references' => '',
//            'value_fulfillment_order' => '',
//            'value_fulfillment_money' => '',
//            'fulfillment_unit' => '',
//            'fulfillment_references' => '',
//            'fulfillment_order' => '',
//            'fulfillment_money' => '',
//            'classification' => '',
            'year' => 'YEAR(date)',
            'month' => 'MONTH(date)',
        ];
    }
    
    /**
     * Combina los parametros y ejecuta el query
     * 
     * @param array $listDefault
     * @param array $params
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _queryExec($listDefault, $params){
        $arrayParams = explode(',', $listDefault);
        $helperReport = new HelperReport($listDefault);
        $tmpSelect = $helperReport->getListParams($this->_paramsSelect($listDefault));
        $select = implode(',', $tmpSelect);
        $tmpGroupBy = $helperReport->getListParams($this->_paramsGroupBy($listDefault));
        $groupBy = 'GROUP BY '.implode(',', $tmpGroupBy);
        $tmpOrderBy = $helperReport->getListParams($this->_paramsOrderBy($listDefault));
        $orderBy = 'ORDER BY '.implode(',', $tmpOrderBy);
        
        $tmpWhere = [];
        $tmpHaving = [];
        $dateEnd = date('Y-m-d');                
        $dateIn = date('Y-m-d', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $this->load($params);
        //Condiciones where ----------------------------------------------------   
        if($this->operation_center != "") 
            $tmpWhere[] = " operation_center like '%$this->operation_center%'"; 
        if($this->type != "") 
            $tmpWhere[] = " type = '$this->type'";
        if($this->date != ""){             
            list($dateIn, $dateEnd) = explode(' - ', $this->date);
            //$tmpWhere[] = " date BETWEEN '$dateIn' AND '$dateEnd'";                      
        }else{            
            if($this->month == '' && $this->year == ''){
                $dateEnd = date('Y-m-d');                
                $dateIn = date('Y-m-d', mktime(0, 0, 0, date('m'), 1, date('Y')));            
                //$tmpWhere[] = " date BETWEEN '$dateIn' AND '$dateEnd'";                      
            }         
        }
        if($this->value_fulfillment_unit != "") 
            $tmpHaving[] = " quantity = '$this->value_fulfillment_unit'";
        if($this->value_fulfillment_references != "") 
            $tmpHaving[] = " value_fulfillment_references = '$this->value_fulfillment_references'";
        if($this->value_fulfillment_order != "") 
            $tmpHaving[] = " value_fulfillment_order = '$this->value_fulfillment_order'";
        if($this->value_fulfillment_money != "") 
            $tmpHaving[] = " value_fulfillment_money = '$this->value_fulfillment_money'";       
        if($this->month != "" && !empty($this->month)){
            $dateIn = date('Y-m-d', mktime(0, 0, 0, (min($this->month)+1), 1, date('Y')));
            $dateEnd = date('Y-m-t', mktime(0, 0, 0, (max($this->month)+1), 1, date('Y')));
            $tempMes = [];
            if(is_array($this->month) && count($this->month)>0){
                foreach ($this->month as $value){
                   $tempMes[] = $value +1; 
                }
                $tmpWhere[] = " MONTH(date) IN('"
                        .implode("','",$tempMes)."')";
            }                                       
        }                                       
        if($this->year != "" && !empty($this->year)){
            $dateIn = date('Y-m-d', mktime(0, 0, 0, 1, 1, $this->year));
            $dateEnd = date('Y-m-t', mktime(0, 0, 0, 12, 31, $this->year));
            $tempAnio = explode(',', $this->year); 
            $tmpWhere[] = " YEAR(date) IN('"
                    .implode("','",$tempAnio)."')"; 
        }
                 
        if(in_array('type', $arrayParams)){
            $tmpWhere[] = " type IN ('".Fulfillmentprovision::PENDINGS."',"
                    . "'".Fulfillmentprovision::BILLS."',"
                    . "'".Fulfillmentprovision::ORDERS."',"
                    . "'".Fulfillmentprovision::ORDERS_PENDINGS."',"
                    . "'".Fulfillmentprovision::ORDERS_FULL."',"
                    . "'".Fulfillmentprovision::ORDERS_TOTAL."')";
            
//            $arrayResult = ReportSiesa::reportFulfillmentProvision($select, $where, 
//                $groupBy, $orderBy, $having, $dateIn, $dateEnd, true);
        }else{
            $tmpWhere[] = " type IN ('".Fulfillmentprovision::BILLS."',"
                    . "'".Fulfillmentprovision::ORDERS_FULL."')";
            
//           $arrayResult = ReportSiesa::reportFulfillmentProvision($select, $where, 
//                $groupBy, $orderBy, $having, $dateIn, $dateEnd, false); 
        }
        
        $where = (count($tmpWhere)>0) ? ' WHERE '.implode(" AND ", $tmpWhere): "";
        $having = (count($tmpHaving)>0) ? 'HAVING '.implode(" AND ", $tmpHaving): "";
        $arrayResult = DataFulfillmentProvision::getData($select, $where, 
                $groupBy, $orderBy, $having, $dateIn, $dateEnd);

        return $arrayResult;
    }
    
    /**
     * Genera los  resultados dinamicos para el reporte
     * 
     * @param array $listDefault
     * @param array $params
     * @param array $preview
     * 
     * @return \yii\data\ArrayDataProvider
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function reportFulfillment($listDefault, $params, $preview = false) {
        $arrayResult = $this->_queryExec($listDefault, $params);   
        # Validamos si se presentan todos los datos ----------------------------
        $paramShowAll = '_tog';
        $exportAll = 'all';
        $pagination = ['pageSize' => 30];
        foreach ($_GET as $key => $value) {
            if (strpos($key, $paramShowAll) !== false) {
                if ($value == $exportAll) {
                    $pagination = false;
                }
            }
        }        
        # ----------------------------------------------------------------------        
        if($preview){
            $sort = false;
        }else{
            $sort = [ 'attributes' =>explode(',', $listDefault)];
        }        
        # ----------------------------------------------------------------------        
        return new ArrayDataProvider([
            'allModels' => $arrayResult,
            'sort' => $sort,
            'pagination' => $pagination
        ]);
    }
    
    /**
     * Genera los datos para el grafico dinamico
     * 
     * @param type $dataEjex
     * @param type $dataEjey
     * @param type $params
     * 
     * @return \yii\data\ArrayDataProvider
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function reportGraph($dataEjex, $dataEjey, $params, $type) {
        $listDefault = $dataEjex.','.$dataEjey;
        $months = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
        # ----------------------------------------------------------------------
        $listParams = $this->listParams();
        $result = $this->_queryExec($listDefault, $params); 
        # ----------------------------------------------------------------------         
        $groupEjex = explode(',', $dataEjex);
        $keyInit = $groupEjex[0];
        unset($groupEjex[0]);
        if(count($groupEjex)>0){
            $arrayResult = ArrayHelper::index($result, $keyInit, $groupEjex);
        }else{
            $arrayResult = ArrayHelper::index($result, $keyInit);
        } 
        #-----------------------------------------------------------------------
        $data = [];
        $data['x'] = [];
        $data['y'] = [];
        $categories = [];
        
        if(count($arrayResult)>0){ 
            $resultsDataY = [];
            $tempEjeY = explode(',', $dataEjey);
            $helperReport = new HelperReport;
            $helperReport->recursiveWalk($categories, $arrayResult, $resultsDataY, $tempEjeY);
            #-------------------------------------------------------------------
            foreach ($resultsDataY as $key => $value) { 
                switch ($key) {
                    case ('value_fulfillment_unit'):
                    case ('value_fulfillment_references'):
                    case ('value_fulfillment_order'):
                    case ('fulfillment_unit'):
                    case ('fulfillment_references'):
                    case ('fulfillment_order'):
                    case ('fulfillment_money'):
                    case ('fulfillment_total'):
                        $name = 'Suma('.$listParams[$key]['content'].')'; 
                        break;
                    default:
                        $name = 'Cant('.$listParams[$key]['content'].')';                        
                        break;
                }
                $data['y'][] = [
                    'type' => $type,  
                    'name'=>$name, 
                    'data'=> array_column($value, $key)];
            }
        } 
        #-----------------------------------------------------------------------
        $response = [
            'lang' => [
                'downloadJPEG' => 'Descargar en JPG',
                'downloadPDF' => 'Descargar en PDF',
                'downloadPNG' => 'Descargar en PNG',
                'downloadSVG' => 'Descargar en SVG',
                'printChart' => 'Imprimir',
                'resetZoom' => 'Reestablecer',
                'resetZoomTitle' => 'Reestablecer 1:1',
            ],
            'chart' => [
                'renderTo' => 'chart-graficas',   
                'height' => '800',
                'zoomType' => 'xy',      
                'responsive'=>[
                    'rules'=>[
                        'condition'=>[
                            //'maxWidth'=>1200
                        ],
                        'chartOptions'=>[
                            'legend'=>['enabled'=>false]
                        ]
                    ]
                ]
            ],
            'exporting' => [
                //'scale' => '4',   
                //'width' => '900',   
                'sourceWidth' => '1000',   
                //'printMaxWidth' => '1000',   
            ],
            'id' => 'chart-g',
            'title' => ['text' => NULL],            
            'plotOptions' => [
                'column' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],
                    'center' => ['50%', '50%'],                    
                    'enableMouseTracking' => false,                    
                ],               
                'area' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],                    
                ],                                            
                'line' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],                    
                ],                                            
            ],
            'xAxis' => [
                //'title' => ['text' => $listParams[$dataEjex]['content']],
                'labels' => ['rotation' => '-90'],
                'categories' => $categories,               
            ],
            'legend'=>[
                    'layout' => 'vertical',
                    'align' => 'left',
                    'verticalAlign' => 'middle'
            ],
            'yAxis' => [
               'title' => ['text' => '']
            ],
            'series' => $data['y']
        ];
        return json_encode($response, JSON_NUMERIC_CHECK);
    }
}
