<?php

namespace app\models;

use Yii;

/**
 * This is the model class for table "items".
 *
 * @property integer $id
 * @property string $reference
 * @property string $description
 * @property double $quantity_sold
 * @property double $quantity_inventory
 * @property double $quantity_committed
 * @property double $quantity_transit
 * @property string $unit_value
 * @property string $provider_id
 * @property string $branch_id
 * @property string $cellar_id
 * @property string $abc
 * @property string $measured_unit
 * @property string $deviation_inventory
 * @property string $deviation_sales
 * @property string $deviation_purchases
 * @property string $deviation_orders
 * @property string $average_inventory
 * @property string $average_sales
 * @property string $average_purchases
 * @property string $average_orders
 * @property string $created
 */
class Items extends \yii\db\ActiveRecord
{
    public $coverage;
    public $quantity_subjected;
    public $consumption;
    public $per_partipipation;
    public $accumulated;
    public $net;
    public $lost_sale;
    public $name_reference;
    
    const CAT_A = 'A';
    const CAT_B = 'B';
    const CAT_C = 'C';
    const UNID = 'UNID';
    
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'items';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['reference', 'description', 'quantity_sold', 'quantity_inventory', 
                'quantity_committed', 'quantity_transit', 'unit_value', 
                'provider_id', 'branch_id', 'cellar_id', 'created'], 'required'],
            [['quantity_sold', 'quantity_inventory', 'quantity_committed', 
                'quantity_transit', 'per_partipipation', 'accumulated', 
                'deviation_inventory', 'deviation_sales', 
                'deviation_purchases', 'deviation_orders', 'average_inventory', 
                'average_sales', 'average_purchases', 'average_orders', 
                'variation_inventory', 'variation_sales', 'variacion_purchases',
                'variation_orders'], 'number'],
            [['unit_value'], 'integer'],
            [['created'], 'safe'],
            [['reference', 'provider_id', 'branch_id', 'cellar_id', 'measured_unit'], 'string', 'max' => 45],
            [['description'], 'string', 'max' => 250],
            [['abc'], 'string', 'max' => 1],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'reference' => Yii::t('app', 'Reference'),
            'description' => Yii::t('app', 'Description'),
            'quantity_sold' => Yii::t('app', 'Quantity Sold'),
            'quantity_inventory' => Yii::t('app', 'Quantity Inventory'),
            'quantity_committed' => Yii::t('app', 'Quantity Committed'),
            'quantity_transit' => Yii::t('app', 'Quantity Transit'),
            'unit_value' => Yii::t('app', 'Unit Value'),
            'provider_id' => Yii::t('app', 'Provider ID'),
            'branch_id' => Yii::t('app', 'Branch ID'),
            'cellar_id' => Yii::t('app', 'Cellar ID'),
            'abc' => Yii::t('app', 'Abc'),
            'measured_unit' => Yii::t('app', 'Measured Unit'),
            'created' => Yii::t('app', 'Created'),
            'coverage' => Yii::t('app', 'Coverage'),
            'quantity_subjected' => Yii::t('app', 'Quantity Subjected'),
            'variation' => Yii::t('app', 'Variation'),
            'consumption' => Yii::t('app', 'Consumption'),
            'deviation_inventory' => Yii::t('app', 'Deviation Inventory'),
            'deviation_sales' => Yii::t('app', 'Deviation Sales'),
            'deviation_purchases' => Yii::t('app', 'Deviation Purchases'),
            'deviation_orders' => Yii::t('app', 'Deviation Orders'),
            'average_inventory' => Yii::t('app', 'Average Inventory'),
            'average_sales' => Yii::t('app', 'Average Sales'),
            'average_purchases' => Yii::t('app', 'Average Purchases'),
            'average_orders' => Yii::t('app', 'Average Orders'),
            'variation_inventory' => Yii::t('app', 'Variation Inventory'),
            'variation_sales' => Yii::t('app', 'Variation Sales'),
            'variation_purchases' => Yii::t('app', 'Variation Purchases'),
            'variation_orders' => Yii::t('app', 'Variation Orders'),
            'lost_sale' => Yii::t('app', 'Lost Sale'),
        ];
    }
    
    /**
     * Retorna el tipo de movimeinto del item 
     * 
     * @param double $desviation
     * @param double $average
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getStrDesviation($desviation, $average){
        $indexAvg = ($average > 0)? $desviation/$average: 0;
        $indexAvg = $indexAvg*100;
        return ($indexAvg <= 20 )? Yii::t('app', 'Regular')
                :(($indexAvg > 20 && $indexAvg < 50 )? Yii::t('app', 'Semirregular') 
                : Yii::t('app', 'Irregular') );
    }
}
