<?php

namespace app\models;

use Yii;
use yii\base\Model;

/**
 * LoginForm is the model behind the login form.
 *
 * @property User|null $user This property is read-only.
 *
 */
class LoginForm extends Model
{
    public $username;
    public $password;
    public $rememberMe = true;
    public $email;

    private $_user = false;


    /**
     * @return array the validation rules.
     */
    public function rules()
    {
        return [
            // username and password are both required
            [['username', 'password'], 'required'],
            [['email'], 'required', 'on' => ['scnrepass']],
            [['email'], 'exist', 'skipOnError' => true, 'targetClass' => Cmpuser::className(), 'targetAttribute' => ['email' => 'email'], 'on' => ['scnrepass']],
            ['email', 'email', 'message' => 'Ingrese un correo electrónico válido.'],
            // rememberMe must be a boolean value
            ['rememberMe', 'boolean'],
            // password is validated by validatePassword()
            ['password', 'validatePassword'],
        ];
    }
    
    public function attributeLabels()
    {
        return [
            'username' => Yii::t('app', 'username'),
            'password' => Yii::t('app', 'password'),
            'rememberMe' => Yii::t('app', 'rememberMe'),            
            'email' => Yii::t('app', 'Correo'),            
        ];
    }

    public function scenarios()
    {
        $scenarios = parent::scenarios();        
        $scenarios['scnrepass'] = ['email'];        
        return $scenarios;
    }
    
    /**
     * Validates the password.
     * This method serves as the inline validation for password.
     *
     * @param string $attribute the attribute currently being validated
     * @param array $params the additional name-value pairs given in the rule
     */
    public function validatePassword($attribute, $params)
    {
        if (!$this->hasErrors()) {
            $user = $this->getUser();

            if (!$user || !$user->validatePassword($this->password)) {
                $this->addError($attribute, 'Incorrect username or password.');
            }
        }
    }

    /**
     * Logs in a user using the provided username and password.
     * @return bool whether the user is logged in successfully
     */
    public function login()
    {
        if ($this->validate()) {
            return Yii::$app->user->login($this->getUser(), $this->rememberMe ? 3600*24*30 : 0);
        }
        return false;
    }

    /**
     * Finds user by [[username]]
     *
     * @return User|null
     */
    public function getUser()
    {
        if ($this->_user === false) {
            $this->_user = User::findByUsername($this->username);
        }

        return $this->_user;
    }
    
    public function sendMailRePass() {
        $npass = substr(uniqid('', true), 0, 10);
        $scheme = isset($_SERVER['HTTPS']) ? 'https://' : 'http://';
        $url = $scheme . $_SERVER['SERVER_NAME'] . Yii::getAlias('@web');        
        $query = 'SELECT * FROM usuarios WHERE email = "'.$this->email.'"';
        $usuario = Yii::$app->db->createCommand($query)->queryOne();         
        if (!empty($usuario)) {            
            try {
                $query = 'UPDATE usuarios SET password="'.md5($npass).'" WHERE email = "'.$this->email.'"';                
                Yii::$app->db->createCommand($query)->execute();
                $body = Yii::$app->controller->renderPartial('mailRePass',
                    ['npass' => $npass,
                'username' => $usuario['username'], 'url' => $url]);
                Yii::$app->mailer->compose()
                        ->setTo($this->email)
                        ->setFrom(['ingeneo@ingeneo.com.co' => 'Action Button Argos'])
                        ->setSubject(Yii::t('app', 'subjet_repass_mail'))
                        ->setHtmlBody($body)
                        ->attach($url . '/img/logo-argos-bn.png')
                        ->send();
                Yii::$app->session->setFlash('success',
                        Yii::t('app', 'success_mail') . $this->email);
                $this->email = '';
            } catch (\Exception $exc) {                
                \Yii::error($exc->getMessage(), 'mail');
                Yii::$app->session->setFlash('error',
                        Yii::t('app', 'error_mail'));
            }
        }else{
            Yii::$app->session->setFlash('error', Yii::t('app', 'email_no_exist'));
        }
    }
}
