<?php

namespace app\models;

use Yii;
use app\components\ReportSiesa;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "pending".
 *
 * @property int $id
 * @property string $nit
 * @property string $business_name
 * @property string $branch_code
 * @property string $branch_name
 * @property string $number_order
 * @property string $reference
 * @property string $description
 * @property double $quantity
 * @property int $cost
 * @property string $reason
 * @property string $detail
 * @property string $responsable
 * @property string $created
 * @property string $created_by
 * @property string $updated
 * @property string $updated_by
 */
class Pending extends BeforeModel
{
    public $reasonId;
    
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'pending';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['nit', 'business_name', 'branch_code', 'branch_name', 
                'number_order','reference', 'description', 'quantity', 
                'cost'], 'required'],
            [['quantity','nit', 'reasonId'], 'number'],
            [['cost'], 'integer'],
            [['detail','operation_center'], 'string'],
            [['created', 'updated'], 'safe'],
            [['number_order'], 'string', 'max' => 100],
            [['reference', 'branch_code'], 'string', 'max' => 45],
            [['business_name','branch_name','description', 'reason', 'responsable'], 'string', 'max' => 250],
            [['created_by', 'updated_by'], 'string', 'max' => 150],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'nit' => Yii::t('app', 'Nit'),
            'business_name' => Yii::t('app', 'Business Name'),
            'branch_code' => Yii::t('app', 'Branch Code'),
            'branch_name' => Yii::t('app', 'Branch Name'),
            'number_order' => Yii::t('app', 'Number Order'),
            'reference' => Yii::t('app', 'Reference'),
            'description' => Yii::t('app', 'Description'),
			'operation_center' => Yii::t('app', 'Operation Center'),
            'quantity' => Yii::t('app', 'Quantity'),
            'cost' => Yii::t('app', 'Cost'),
            'reasonId' => Yii::t('app', 'Reason'),
            'reason' => Yii::t('app', 'Reason'),
            'detail' => Yii::t('app', 'Detail'),
            'responsable' => Yii::t('app', 'Responsable'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated' => Yii::t('app', 'Updated'),
            'updated_by' => Yii::t('app', 'Updated By'),
        ];
    }
    
    /**
     * Retorna los productos asociados a un nit y sucursal
     * 
     * @param string $nit         Nit de la sucursal
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getPendings(){
        $tmpWhere = [];
        $tmpBDWhere = [];
        if($this->reference != "") 
            $tmpWhere[] = "f430_id_tipo_docto = '$this->reference'";
            $tmpBDWhere[] = "reference = '$this->reference'";
        if($this->description != "") 
            $tmpWhere[] = "f350_id_tipo_docto = '$this->description'";
            $tmpBDWhere[] = "description = '$this->description'";
        if($this->created != ""){             
            list($dateIn, $dateEnd) = explode(' - ', $this->created);
            $tmpWhere[] = " PV.f430_id_fecha BETWEEN '$dateIn' AND '$dateEnd'";                      
            $tmpBDWhere[] = " created BETWEEN '$dateIn' AND '$dateEnd'";                      
        }else{            
            $dateEnd = date('d-m-Y');                
            //$dateIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));            
            $dateIn = date('d-m-Y');            
            $tmpWhere[] = " PV.f430_id_fecha BETWEEN '$dateIn' AND '$dateEnd'";                      
            $tmpBDWhere[] = " created BETWEEN '$dateIn' AND '$dateEnd'";                      
        }
        $siesa = new ReportSiesa;
        $where = ' AND '.implode(' AND ', $tmpWhere);
        $whereBD = implode(' AND ', $tmpBDWhere);
        
        $bdReferences = self::findAll($whereBD);
        $tmpReferences = ArrayHelper::index($bdReferences, 'reference');
        
        $references = $siesa->getPendings($where);
        $arryaData = [];
        if(count($references)>0){
            foreach ($references as $reference) {
                if(isset($tmpReferences[$reference['reference']])){
                    $tempData = $tmpReferences[$reference['reference']];
                }else{
                    $tempData = new Pending();
                }
                $tempData->reference = $reference['reference'];
                $tempData->description = $reference['description'];
                $tempData->cost = (int)$reference['total_cost'];
                $tempData->quantity = (int)$reference['pending'];
                $tempData->save();
                $arryaData[] = $tempData;
            }
        }
        return $arryaData;
    }  
}
