<?php

namespace app\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;
use app\models\Pending;
use yii\data\ArrayDataProvider;

/**
 * PendingSearch represents the model behind the search form of `app\models\Pending`.
 */
class PendingSearch extends Pending
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'cost'], 'integer'],
            [['nit', 'business_name', 'branch_code', 'branch_name', 
                'number_order','reference', 'description', 'reason', 
                'detail', 'responsable', 'created', 'created_by', 'updated', 
                'updated_by', 'operation_center'], 'safe'],
            [['quantity', 'nit'], 'number'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = Pending::find();
        $query->orderBy('created desc');

        // add conditions that should always apply here
		
		
		$all = false;
        foreach ($_REQUEST as $key => $value) {
            if (strpos($key, '_tog') !== false) {
                if ($value == 'all') {
                    $all = true;
                }
            }
        }
		
        
        if(isset($_POST['PendingSearch'])){
            $post['PendingSearch'] = $_POST['PendingSearch'];
			if($all){
				$pagination = false;
			}else{
				$pagination = ['pageSize' => 20, 'params'  => $post];
			}

        }else{
			if($all){
				$pagination = false;
			}else{
				$pagination = ['pageSize' => 20];
			}
        }
		
		$dataProvider = new ActiveDataProvider([
                'query' => $query,
				'pagination'=>$pagination,
				
            ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        $this->_filter($query);

        return $dataProvider;
    }
	
	/**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function searchExport($params)
    {
        $query = Pending::find();
        $query->orderBy('created desc');

        // add conditions that should always apply here
       
        
		if(isset($_POST['PendingSearch'])){
            $post['PendingSearch'] = $_POST['PendingSearch'];
			$this->load($post);
        }else{
            $this->load($params);
        }

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        $this->_filter($query);
		
		$pagination = ['pageSize' => 20];
        foreach ($_REQUEST as $key => $value) {
            if (strpos($key, '_tog') !== false) {
                if ($value == 'all') {
                    $pagination = false;
                }
            }
        }
		
		$dataProvider = new ArrayDataProvider([
                'allModels' => $query->asArray()->all(),
				'pagination'=>$pagination,
            ]);

        return $dataProvider;
    }
    
    /**
     * Filtros del search
     * 
     * @param object $query
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _filter(&$query){
        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'quantity' => $this->quantity,
            'cost' => $this->cost,
            'updated' => $this->updated,
        ]);
        
        $query
            ->andFilterWhere(['like', 'nit', $this->nit])
            ->andFilterWhere(['like', 'business_name', $this->business_name])
            ->andFilterWhere(['like', 'branch_code', $this->branch_code])
            ->andFilterWhere(['like', 'branch_name', $this->branch_name])
            ->andFilterWhere(['like', 'number_order', $this->number_order])
            ->andFilterWhere(['like', 'reference', $this->reference])
            ->andFilterWhere(['like', 'description', $this->description])
			->andFilterWhere(['like', 'operation_center', $this->operation_center])
            ->andFilterWhere(['like', 'reason', $this->reason])
            ->andFilterWhere(['like', 'detail', $this->detail])
            ->andFilterWhere(['like', 'responsable', $this->responsable])
            ->andFilterWhere(['like', 'created_by', $this->created_by])
            ->andFilterWhere(['like', 'updated_by', $this->updated_by]);
        
        if(!empty($this->created) && strpos($this->created, ' - ')){
            list($dateIn, $dateEnd) = explode(' - ', $this->created);
            $query
            ->andFilterWhere(['between', 'created', $dateIn, $dateEnd]);
        }else{
           $query->andFilterWhere([
                'created' => $this->created,
            ]); 
        }
    }
    
    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return data
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function searchGraph($params)
    {
        $query = Pending::find()
                ->select('reason, count(*) as quantity')
                ->groupBy('reason')
				->orderBy('created desc');

        $this->load($params);

        if (!$this->validate()) {
            return [];
        }
        
        $this->_filter($query);
        
        $result = $query->asArray()->all();
        $data = [];
        $data['x'] = [];
        $data['categories'] = [];
        $data['y'] = [];
        if(count($result)>0){
            $total = 0;
            foreach ($result as $data) {
                $total += $data['quantity'];
            }
            $dataGraph = [];
            $categories = [];
            foreach ($result as $data) {
                if(empty($data['reason'])){
                    $name = \Yii::t('app', 'Without reason');
                }else{
                    $name = $data['reason'];
                }
                $categories[] = $name;
                $dataGraph[] = ((int)$data['quantity']*100/$total);
            }
            $data['categories'] = $categories;
            $data['y'] = $dataGraph; 
        }
        
        return $data;
    }
}
