<?php

namespace app\models;

use Yii;
use app\components\purchases\ClientSiesa;
use yii\helpers\ArrayHelper;
use yii\db\ActiveRecord;
use yii\behaviors\BlameableBehavior;
use yii\db\Expression;

/**
 * This is the model class for table "purchases".
 *
 * @property string $id
 * @property string $nit
 * @property string $business_name
 * @property string $branch_code
 * @property string $branch_name
 * @property string $cellar
 * @property string $total
 * @property string $created
 * @property string $created_by
 * @property string $updated
 * @property string $updated_by
 *
 * @property ItemsPurchase[] $itemsPurchases
 */
class Purchases extends ActiveRecord
{
    const STATUS_PENDING = 'pendiente';
    const STATUS_ALLOW = 'autorizado';
    const STATUS_PARCIAL = 'parcial';
    const STATUS_DELETE = 'borrado';
    
    public $projection_day;
    public $start_date;
    public $end_date;
    public $items;
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'purchases';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['nit', 'business_name', 'branch_code', 'branch_name', 
                'cellar', 'projection_day', 'start_date', 'end_date'], 'required'],
            [['id', 'nit', 'total', 'projection_day'], 'integer'],
            [['created', 'updated'], 'safe'],
            [['business_name', 'branch_name'], 'string', 'max' => 250],
            [['branch_code', 'cellar'], 'string', 'max' => 45],
            [['created_by', 'updated_by'], 'string', 'max' => 150],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'nit' => Yii::t('app', 'Nit'),
            'business_name' => Yii::t('app', 'Business Name'),
            'branch_code' => Yii::t('app', 'Branch Code'),
            'branch_name' => Yii::t('app', 'Branch Name'),
            'cellar' => Yii::t('app', 'Cellar'),
            'total' => Yii::t('app', 'Total'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated' => Yii::t('app', 'Updated'),
            'updated_by' => Yii::t('app', 'Updated By'),
            'projection_day' => Yii::t('app', 'Projection day'),
            'start_date' => Yii::t('app', 'Start day'),
            'end_date' => Yii::t('app', 'End day'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getItemsPurchases()
    {
        return $this->hasMany(ItemsPurchase::className(), ['purchase_id' => 'id']);
    }
    
    /**
     * Retorna el listados de proveedores para el dropdown dle formulario 
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public static function selectProviders($strProvider) {
        $clientSiesa = new ClientSiesa;
        $providers = $clientSiesa->getProviders($strProvider);
        $data = [];
        if(count($providers)>0){                   
            foreach ($providers as $provider) {
                $data[] = ['id' => $provider['nit'], 'text' => $provider['business_name']];  
            }                 
        } 
        return $data;
    }
    
    /**
     * Retorna el listados de sucursales para el dropdown dle formulario 
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public static function selectBranches($nit) {
        $clientSiesa = new ClientSiesa;
        $branches = $clientSiesa->getBranches($nit);
        $data = [];
        if(count($branches)>0){                   
            foreach ($branches as $branch) {
                $data[] = ['id' => $branch['branch_code'], 'name' => $branch['branch_name']];  
            }                 
        } 
        return $data;
    }
    
    /**
     * Retorna el listados de bodegas para el dropdown dle formulario 
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public static function selectCellars() {
        $clientSiesa = new ClientSiesa;
        $cellars = $clientSiesa->getCellars();
        $data = [];
        
        if(count($cellars)>0){ 
            for ($i = 0; $i < sizeof($cellars); $i++) {
                $data[$cellars[$i]['cellar_code']] = $cellars[$i]['cellar_name'];
            }               
        } 
        return $data;
    }
    
    /**
     * Diferencia de dias 
     * 
     * @param date $date1
     * @param date $date2
     * 
     * @return int
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function dateDiff($date1, $date2) {
        $diff = 0;
        if (!empty($date1) && !empty($date2)) {
            $datetime1 = date_create($date1);
            $datetime2 = date_create($date2);
            $interval = date_diff($datetime1, $datetime2);            
            $diff = $interval->format('%a');
        }
        return $diff;
    }
    
    /**
     * Retorna los productos asociados a un nit y sucursal
     * 
     * @param string $nit         Nit de la sucursal
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getItems(){
        $siesa = new ClientSiesa;
        $dateDiff = ($this->dateDiff($this->start_date, $this->end_date)+1);
        return $siesa->getItems($this->nit, $this->branch_code, $this->cellar, 
                $this->start_date, $this->end_date, $dateDiff, $this->projection_day);
    }
    
    /**
     * 
     * @param type $model
     * @return \yii\data\ActiveDataProvider
     */
    public function getListItemsPurchases(){
        $query = Itemspurchase::find()->where(['purchase_id'=>$this->id]);
        return new \yii\data\ActiveDataProvider([
            'query' => $query,
            'pagination' => false,
            'sort' => false,
        ]);
    }
    
}
