<?php

namespace app\models;

use Yii;
use kartik\grid\GridView;
use yii\helpers\ArrayHelper;
use kartik\select2\Select2;

/**
 * This is the model class for table "reportes".
 *
 * @property int $id
 * @property string $report
 * @property string $addParameters
 * @property string $defaultParameters
 * @property string $ejex
 * @property string $ejey
 * @property string $type
 * @property int $cia_id
 * @property int $user_id
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property User $user
 */
class ReportsDynamic extends BeforeModel
{
    public $id;
    public $reference;
    public $description;
    public $provider_id;
    public $provider_name;
    public $branch_office_id;
    public $branch_office_name;
    public $cellar_id;
    public $cellar_name;
    public $subtotal;
    public $margin;
    public $participation;
    public $quantity_existence;
    public $quantity_sold;
    public $quantity_op;
    public $quantity_available;
    public $quantity_previous;
    public $abc;
    public $measured_unit;
    public $last_cost;
    public $arrayKeys;

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'reports';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'reference', 'description' , 'provider_id', 'provider_name', 'branch_office_id',
                'branch_office_name', 'cellar_id', 'cellar_name', 'subtotal', 'margin', 'participation',
                'quantity_existence', 'quantity_sold', 'quantity_op', 'quantity_available', 'abc', 'measured_unit', 'quantity_previous', 'last_cost'], 'safe']];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'report' => Yii::t('app', 'Report'),
            'addParameters' => Yii::t('app', 'Add Parameters'),
            'defaultParameters' => Yii::t('app', 'Default Parameters'),
            'ejex' => Yii::t('app', 'Ejex'),
            'ejey' => Yii::t('app', 'Ejey'),
            'type' => Yii::t('app', 'Tipo'),
            'cia_id' => Yii::t('app', 'Cia'),
            'user_id' => Yii::t('app', 'User'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),
        ];
    }

    /**
     * Listado de los parametros por defecto sin espacios
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listDefaultParameters()
    {
        $list = 'reference,description,provider_name,branch_office_name,subtotal,quantity_existence,quantity_available';
        return str_replace(' ', '', $list);
    }

    /**
     * Listado de los parametros adicionales sin espacios
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParametersAdd()
    {
        $list = 'measured_unit,cellar_id,cellar_name,margin,participation,quantity_sold,quantity_op,abc,quantity_previous,last_cost';
        return str_replace(' ', '', $list);
    }

    /**
     * Retorna el listado a partir de un array y un string dado
     *
     * @param string $string String separado por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getTempList($string)
    {
        return $this->_getDataFromList($string, $this->listParams());
    }

    /**
     * Retorna el array de los parametros que iran el en select de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListSelect($params)
    {
        return $this->_getDataFromList($params, $this->_paramsSelect());
    }

    /**
     * Retorna el array de los parametros que iran el en select de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListSelectGraph($params)
    {
        return $this->_getDataFromList($params, $this->_paramsSelectGraph($params));
    }

    /**
     * Retorna el array de los parametros que iran el en group de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListGroupBy($params)
    {
        return $this->_getDataFromList($params, $this->_paramsGroupBy());
    }

    /**
     * Retorna el array de los parametros que iran el en group de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListGroupByGraph($params)
    {
        return $this->_getDataFromList($params, $this->_paramsGroupByGraph($params));
    }

    /**
     * Retorna el array de los parametros que iran el en group de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListOrderByGraph($params)
    {
        return $this->_getDataFromList($params, $this->_paramsOrderByGraph($params));
    }

    /**
     * Retorna el array de los parametros que iran el en grid de la vista
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getListGrid($params)
    {
        return $this->_getDataFromList($params, $this->listParamsGrid());
    }

    /**
     * Funcion que busca los datos de un string en un array
     *
     * @param string $string Datos separados por comas
     * @param array $list Array donde se buscaran los datos
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getDataFromList($string, $list)
    {
        $array = explode(',', $string);
        $arrayResul = [];
        if (count($array) > 0) {
            foreach ($array as $value) {
                $trimValue = trim($value);
                if (isset($list[$trimValue])) {
                    $arrayResul[$trimValue] = $list[$trimValue];
                }
            }
        }
        return $arrayResul;
    }

    /**
     * Lisrados de parametros configurables en el reporte
     *
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParams()
    {
        return [
            'reference' => ['content' => Yii::t('app', 'Reference')],
            'description' => ['content' => Yii::t('app', 'Description')],
            'provider_id' => ['content' => Yii::t('app', 'Provider ID')],
            'provider_name' => ['content' => Yii::t('app', 'Provider ID')],
            'branch_office_id' => ['content' => Yii::t('app', 'Branch Office')],
            'branch_office_name' => ['content' => Yii::t('app', 'Branch Office')],
            'cellar_id' => ['content' => Yii::t('app', 'Cellar Id')],
            'cellar_name' => ['content' => Yii::t('app', 'Cellar Name')],
            'subtotal' => ['content' => Yii::t('app', 'Subtotal')],
            'margin' => ['content' => Yii::t('app', 'Margin')],
            'participation' => ['content' => Yii::t('app', 'Participation')],
            'quantity_existence' => ['content' => Yii::t('app', 'Quantity Inventory')],
            'quantity_sold' => ['content' => Yii::t('app', 'Quantity Sold')],
            'quantity_op' => ['content' => Yii::t('app', 'Quantity op')],
            'quantity_available' => ['content' => Yii::t('app', 'Quantity Available')],
            'abc' => ['content' => Yii::t('app', 'Abc Margin')],
            'measured_unit' => ['content' => Yii::t('app', 'Measured Unit')],
            'quantity_previous' => ['content' => Yii::t('app', 'Quantity Previous')],
            'last_cost' => ['content' => Yii::t('app', 'Last Cost')]
        ];
    }

    /**
     * Metodo que retorna el listado de las columnas para el grid
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsGrid()
    {
        # Traemos todos los meses para el filtro -------------------------------
        $id_user = Yii::$app->user->identity->getId();

        return [
            'reference' => [
                'attribute' => 'reference',
                'header' => Yii::t('app', 'Reference'),
            ],
            'description' => [
                'attribute' => 'description',
                'header' => Yii::t('app', 'Description'),
            ],
            'provider_name' => [
                'attribute' => 'provider_name',
                'header' => Yii::t('app', 'Provider'),
            ],
            'branch_office_name' => [
                'attribute' => 'branch_office_name',
                'header' => Yii::t('app', 'Branch Office'),
            ],
            'cellar' => [
                'attribute' => 'cellar',
                'header' => Yii::t('app', 'Cellar'),
            ],
            'subtotal' => [
                'attribute' => 'subtotal',
                'header' => Yii::t('app', 'Subtotal'),
            ],
            'cellar_id' => [
                'attribute' => 'cellar_id',
                'header' => Yii::t('app', 'Cellar Id'),
            ],
            'cellar_name' => [
                'attribute' => 'cellar_name',
                'header' => Yii::t('app', 'Cellar Name'),
            ],
            'participation' => [
                'attribute' => 'participation',
                'header' => Yii::t('app', 'Participation'),
            ],
            'quantity_existence' => [
                'attribute' => 'quantity_existence',
                'header' => Yii::t('app', 'Quantity Inventory'),
            ],
            'quantity_sold' => [
                'attribute' => 'quantity_sold',
                'header' => Yii::t('app', 'Quantity Sold'),
            ],
            'quantity_op' => [
                'attribute' => 'quantity_op',
                'header' => Yii::t('app', 'Quantity op'),
            ],
            'quantity_available' => [
                'attribute' => 'quantity_available',
                'header' => Yii::t('app', 'Quantity Available'),
            ],
            'abc' => [
                'attribute' => 'abc',
                'header' => Yii::t('app', 'Abc Margin'),
            ],
            'measured_unit' => [
                'attribute' => 'measured_unit',
                'header' => Yii::t('app', 'Measured Unit'),
            ],
            'quantity_previous' => [
                'attribute' => 'quantity_previous',
                'header' => Yii::t('app', 'Quantity Previous'),
            ],
            'last_cost' => [
                'attribute' => 'last_cost',
                'header' => Yii::t('app', 'Last Cost'),
            ]
        ];
    }

    private function _paramsSelectGraph() {
        # ----------------------------------------------------------------------
        return [
            'reference' => 'reports_references.reference as reference',
            'description' => 'reports_references.description as description',
            'provider_id' => 'reports_references.provider_id as provider_id',
            'provider_name' => 'reports_references.provider_name as provider_name',
            'branch_office_id' => 'reports_references.branch_office_id as branch_office_id',
            'branch_office_name' => 'reports_references.branch_office_name as branch_office_name',
            'cellar_id' => 'reports_references.cellar_id as cellar_id',
            'cellar_name' => 'reports_references.cellar_name as cellar_name',
            'subtotal' => 'reports_references.subtotal as subtotal',
            'margin' => 'reports_references.margin as margin',
            'participation' => 'reports_references.participation as participation',
            'quantity_existence' => 'reports_references.quantity_existence as quantity_existence',
            'quantity_sold' => 'reports_references.quantity_sold as quantity_sold',
            'quantity_op' => 'reports_references.quantity_op as quantity_op',
            'quantity_available' => 'reports_references.quantity_available as quantity_available',
            'abc' => 'reports_references.abc as abc',
            'measured_unit' => 'reports_references.measured_unit as measured_unit',
            'quantity_previous' => 'reports_references.quantity_previous as quantity_previous',
            'last_cost' => 'reports_references.last_cost as last_cost'
        ];
    }

    /**
     * Retorna los parametrso que van en el select del B2BiSiesa
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsSelect() {
        # ----------------------------------------------------------------------
        return [
            'reference' => 'reports_references.reference as reference',
            'description' => 'reports_references.description as description',
            'provider_id' => 'reports_references.provider_id as provider_id',
            'provider_name' => 'reports_references.provider_name as provider_name',
            'branch_office_id' => 'reports_references.branch_office_id as branch_office_id',
            'branch_office_name' => 'reports_references.branch_office_name as branch_office_name',
            'cellar_id' => 'reports_references.cellar_id as cellar_id',
            'cellar_name' => 'reports_references.cellar_name as cellar_name',
            'subtotal' => 'reports_references.subtotal as subtotal',
            'margin' => 'reports_references.margin as margin',
            'participation' => 'reports_references.participation as participation',
            'quantity_existence' => 'reports_references.quantity_existence as quantity_existence',
            'quantity_sold' => 'reports_references.quantity_sold as quantity_sold',
            'quantity_op' => 'reports_references.quantity_op as quantity_op',
            'quantity_available' => 'reports_references.quantity_available as quantity_available',
            'abc' => 'reports_references.abc as abc',
            'measured_unit' => 'reports_references.measured_unit as measured_unit',
            'quantity_previous' => 'reports_references.quantity_previous as quantity_previous',
            'last_cost' => 'reports_references.last_cost as last_cost'
        ];
    }

    /**
     * Retorna los parametrso que van en el select del B2BiSiesa
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsSelectGrafica($params) {
        return [
            'reference' => 'reports_references.reference',
            'description' => 'reports_references.description',
            'provider_id' => 'reports_references.provider_id',
            'provider_name' => 'reports_references.provider_name',
            'branch_office_id' => 'reports_references.branch_office_id',
            'branch_office_name' => 'reports_references.branch_office_name',
            'cellar_id' => 'reports_references.cellar_id',
            'cellar_name' => 'reports_references.cellar_name',
            'subtotal' => 'reports_references.subtotal',
            'margin' => 'reports_references.margin',
            'participation' => 'reports_references.participation',
            'quantity_existence' => 'reports_references.quantity_existence',
            'quantity_sold' => 'reports_references.quantity_sold',
            'quantity_op' => 'reports_references.quantity_op',
            'quantity_available' => 'reports_references.quantity_available',
            'abc' => 'reports_references.abc',
            'measured_unit' => 'reports_references.measured_unit',
            'quantity_previous' => 'reports_references.quantity_previous',
            'last_cost' => 'reports_references.last_cost'
        ];
    }

    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     *
     * @return type
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsGroupBy() {
        return [
            'reference' => 'reports_references.reference',
            'description' => 'reports_references.description',
            'provider_id' => 'reports_references.provider_id',
            'provider_name' => 'reports_references.provider_name',
            'branch_office_id' => 'reports_references.branch_office_id',
            'branch_office_name' => 'reports_references.branch_office_name',
            'cellar_id' => 'reports_references.cellar_id',
            'cellar_name' => 'reports_references.cellar_name',
            'subtotal' => 'reports_references.subtotal',
            'margin' => 'reports_references.margin',
            'participation' => 'reports_references.participation',
            'quantity_existence' => 'reports_references.quantity_existence',
            'quantity_sold' => 'reports_references.quantity_sold',
            'quantity_op' => 'reports_references.quantity_op',
            'quantity_available' => 'reports_references.quantity_available',
            'abc' => 'reports_references.abc',
            'measured_unit' => 'reports_references.measured_unit',
            'quantity_previous' => 'reports_references.quantity_previous',
            'last_cost' => 'reports_references.last_cost'
        ];
    }

    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     *
     * @return type
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsGroupByGraph($params) {
        return [
            'reference' => 'reports_references.reference',
            'description' => 'reports_references.description',
            'provider_name' => 'reports_references.provider_name',
            'branch_office_name' => 'reports_references.branch_office_name',
            'cellar_id' => 'reports_references.cellar_id',
            'cellar_name' => 'reports_references.cellar_name',
            'subtotal' => 'reports_references.subtotal',
            'margin' => 'reports_references.margin',
            'participation' => 'reports_references.participation',
            'quantity_existence' => 'reports_references.quantity_existence',
            'quantity_sold' => 'reports_references.quantity_sold',
            'quantity_op' => 'reports_references.quantity_op',
            'quantity_available' => 'reports_references.quantity_available',
            'measured_unit' => 'reports_references.measured_unit',
            'quantity_previous' => 'reports_references.quantity_previous',
            'last_cost' => 'reports_references.last_cost'
        ];
    }

    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     *
     * @return type
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsOrderByGraph($params) {
        return [
            'reference' => 'reports_references.reference',
            'description' => 'reports_references.description',
            'provider_name' => 'reports_references.provider_name',
            'branch_office_name' => 'reports_references.branch_office_name',
            'cellar_id' => 'reports_references.cellar_id',
            'cellar_name' => 'reports_references.cellar_name',
            'subtotal' => 'reports_references.subtotal',
            'margin' => 'reports_references.margin',
            'participation' => 'reports_references.participation',
            'quantity_existence' => 'reports_references.quantity_existence',
            'quantity_sold' => 'reports_references.quantity_sold',
            'quantity_op' => 'reports_references.quantity_op',
            'quantity_available' => 'reports_references.quantity_available',
            'measured_unit' => 'reports_references.measured_unit',
            'quantity_previous' => 'reports_references.quantity_previous',
            'last_cost' => 'reports_references.last_cost'
        ];
    }

    /**
     * Genera los  resultados dinamicos para el reporte
     *
     * @param type $valueListDefecto
     * @param type $params
     * @param type $preview
     *
     * @return \yii\data\ArrayDataProvider
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function report($valueListDefaultParameters, $params, $preview = false)
    {
        $select = $this->_getListSelect($valueListDefaultParameters);
        $groupBy = $this->_getListGroupBy($valueListDefaultParameters);

        $query = ReportsReferences::find();
        $query->select(implode(',', $select));
        $query->groupBy(implode(',', $groupBy));
        //$query->orderBy(implode(',', $groupBy));

        if ($preview) {
            $query->limit(10);
        }

        $this->load($params);

        $query->andFilterWhere(['like', 'reports_references.reference', $this->reference])
            ->andFilterWhere(['like', 'reports_references.description', $this->description])
            ->andFilterWhere(['like', 'reports_references.provider_name', $this->provider_name])
            ->andFilterWhere(['like', 'reports_references.branch_office_name', $this->branch_office_name])
            ->andFilterWhere(['like', 'reports_references.subtotal', $this->subtotal])
            ->andFilterWhere(['like', 'reports_references.margin', $this->margin])
            ->andFilterWhere(['like', 'reports_references.cellar_id', $this->cellar_id])
            ->andFilterWhere(['like', 'reports_references.cellar_name', $this->cellar_name])
            ->andFilterWhere(['like', 'reports_references.participation', $this->participation])
            ->andFilterWhere(['like', 'reports_references.quantity_existence', $this->quantity_existence])
            ->andFilterWhere(['like', 'reports_references.quantity_sold', $this->quantity_sold])
            ->andFilterWhere(['like', 'reports_references.quantity_available', $this->quantity_available])
            ->andFilterWhere(['like', 'reports_references.abc', $this->abc])
            ->andFilterWhere(['like', 'reports_references.measured_unit', $this->measured_unit])
            ->andFilterWhere(['like', 'reports_references.quantity_previous', $this->quantity_previous])
            ->andFilterWhere(['like', 'reports_references.last_cost', $this->last_cost])
            ->andFilterWhere(['=', 'reports_references.quantity_op', $this->quantity_op]);

        # ----------------------------------------------------------------------
        try{
            $arrayResult = $query->asArray()->all();
        }catch (\Exception $e){
            return new \yii\data\ArrayDataProvider([
                'allModels' => [],
                'sort' => [],
                'pagination' => []
            ]);
        }

        # Validamos si se presentan todos los datos ----------------------------
        $paramShowAll = '_tog';
        $exportAll = 'all';
        $pagination = ['pageSize' => 15];
        foreach ($_GET as $key => $value) {
            if (strpos($key, $paramShowAll) !== false) {
                if ($value == $exportAll) {
                    $pagination = false;
                }
            }
        }
        # ----------------------------------------------------------------------
        $sort = [ 'attributes' =>explode(',', $valueListDefaultParameters)];
        # ----------------------------------------------------------------------
        return new \yii\data\ArrayDataProvider([
            'allModels' => $arrayResult,
            'sort' => $sort,
            'pagination' => $pagination
        ]);
    }

    /**
     * Genera los datos para el grafico dinamico
     *
     * @param type $dataEjex
     * @param type $dataEjey
     * @param type $params
     *
     * @return \yii\data\ArrayDataProvider
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function reportGraph($dataEjex, $dataEjey, $params, $type) {
        $valueListDefect = $dataEjex.','.$dataEjey;
        $listParams = $this->listParams();
        $select = $this->_getListSelectGraph($valueListDefect);
        $groupBy = $this->_getListGroupByGraph($valueListDefect);
        #----------------------------------------------------------------------
        $query = ReportsReferences::find();
        $query->select(implode(',', $select));
        $query->groupBy(implode(',', $groupBy));
        $this->load($params);
        $nombre = 'Gráfica';
        if(isset($params['id'])){
            $this->id = $params['id'];
            $report = Reports::findOne($this->id);
            $nombre  = $report->name;
        }
        $query->andFilterWhere(['like', 'reports_references.reference', $this->reference])
            ->andFilterWhere(['like', 'reports_references.description', $this->description])
            ->andFilterWhere(['like', 'reports_references.provider_name', $this->provider_name])
            ->andFilterWhere(['like', 'reports_references.branch_office_name', $this->branch_office_name])
            ->andFilterWhere(['like', 'reports_references.subtotal', $this->subtotal])
            ->andFilterWhere(['like', 'reports_references.margin', $this->margin])
            ->andFilterWhere(['like', 'reports_references.cellar_id', $this->cellar_id])
            ->andFilterWhere(['like', 'reports_references.cellar_name', $this->cellar_name])
            ->andFilterWhere(['like', 'reports_references.participation', $this->participation])
            ->andFilterWhere(['like', 'reports_references.quantity_existence', $this->quantity_existence])
            ->andFilterWhere(['like', 'reports_references.quantity_sold', $this->quantity_sold])
            ->andFilterWhere(['like', 'reports_references.quantity_available', $this->quantity_available])
            ->andFilterWhere(['like', 'reports_references.abc', $this->abc])
            ->andFilterWhere(['like', 'reports_references.measured_unit', $this->measured_unit])
            ->andFilterWhere(['like', 'reports_references.quantity_previous', $this->quantity_previous])
            ->andFilterWhere(['like', 'reports_references.last_cost', $this->last_cost])
            ->andFilterWhere(['=', 'reports_references.quantity_op', $this->quantity_op]);
        #-----------------------------------------------------------------------
        try{
            $result = $query->asArray()->all();
        }catch (\Exception $e){
            return new \yii\data\ArrayDataProvider([
                'allModels' => [],
                'sort' => [],
                'pagination' => []
            ]);
        }

        $groupEjex = explode(',', $dataEjex);
        $keyInit = $groupEjex[0];
        unset($groupEjex[0]);
        if(count($groupEjex)>0){
            $arrayResult = ArrayHelper::index($result, $keyInit, $groupEjex);
        }else{
            $arrayResult = ArrayHelper::index($result, $keyInit);
        }
        #-----------------------------------------------------------------------
        $data = [];
        $data['x'] = [];
        $data['y'] = [];
        $categories = [];

        if(count($arrayResult)>0){
            $resultsDataY = [];
            $tempEjeY = explode(',', $dataEjey);
            $this->_recursiveWalk($categories, $arrayResult, $resultsDataY, $tempEjeY);
            #-------------------------------------------------------------------
            foreach ($resultsDataY as $key => $value) {
                switch ($key) {
                    case ('subtotal'):
                        $name = 'Suma('.$listParams[$key]['subtotal'].')';
                        break;
                    case ('quantity_existence'):
                        $name = 'Suma('.$listParams[$key]['content'].')';
                        break;
                    case ('quantity_sold'):
                        $name = 'Suma('.$listParams[$key]['content'].')';
                        break;
                    case ('quantity_op'):
                        $name = 'Suma('.$listParams[$key]['content'].')';
                        break;
                    case ('quantity_available'):
                        $name = 'Suma('.$listParams[$key]['content'].')';
                        break;
                    default:
                        $name = 'Cant('.$listParams[$key]['content'].')';
                        break;
                }
                $data['y'][] = [
                    'type' => $type,
                    'name'=>$name,
                    'data'=> array_column($value, $key)];
            }
        }
        #-----------------------------------------------------------------------
        $response = [
            'lang' => [
                'downloadJPEG' => 'Descargar en JPG',
                'downloadPDF' => 'Descargar en PDF',
                'downloadPNG' => 'Descargar en PNG',
                'downloadSVG' => 'Descargar en SVG',
                'printChart' => 'Imprimir',
                'resetZoom' => 'Reestablecer',
                'resetZoomTitle' => 'Reestablecer 1:1',
            ],
            'chart' => [
                'renderTo' => 'chart-graficas',
                'height' => '800',
                'zoomType' => 'xy',
                'responsive'=>[
                    'rules'=>[
                        'condition'=>[
                            //'maxWidth'=>1200
                        ],
                        'chartOptions'=>[
                            'legend'=>['enabled'=>false]
                        ]
                    ]
                ]
            ],
            'exporting' => [
                //'scale' => '4',
                //'width' => '900',
                'sourceWidth' => '1000',
                //'printMaxWidth' => '1000',
            ],
            'id' => 'chart-g',
            'title' => ['text' => $nombre],
            'plotOptions' => [
                'column' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],
                    'center' => ['50%', '50%'],
                    'enableMouseTracking' => false,
                ],
                'area' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],
                ],
                'line' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],
                ],
            ],
            'xAxis' => [
                //'title' => ['text' => $listParams[$dataEjex]['content']],
                'labels' => ['rotation' => '-90'],
                'categories' => $categories,
            ],
            'legend'=>[
                    'layout' => 'vertical',
                    'align' => 'left',
                    'verticalAlign' => 'middle'
            ],
            'yAxis' => [
               'title' => ['text' => '']
            ],
            'series' => $data['y']
        ];
        return json_encode($response, JSON_NUMERIC_CHECK);
    }

    /**
     * Funcion recursiva para extraer los datos
     *
     * @param array $categories
     * @param array $datos
     * @param array $resultsDataY
     * @param array $tempEjeY
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _recursiveWalk(&$categories, $datos, &$resultsDataY, $tempEjeY){
        $objAux = [];
        $i = 0;
        foreach($datos as $key => $dato){
            if(isset($dato[$tempEjeY[0]])){
                $categories[] = $key;
                foreach ($tempEjeY as $value) {
                    $resultsDataY[$value][][$value] = $dato[$value];
                }
            } else {
                $objAuxJson = [];
                $this->_recursiveWalk($objAuxJson, $dato, $resultsDataY, $tempEjeY);
                $objAux['name'] = $key;
                $objAux['categories'] = $objAuxJson;
                $categories[] = $objAux;
            }
            if(++$i == 5) {break;}
        }
    }
}
