<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\behaviors\BlameableBehavior;
use yii\db\Expression;
use app\components\purchases\ClientSiesa;

/**
 * This is the model class for table "times_delivery".
 *
 * @property integer $id
 * @property string $oc_code
 * @property string $oc_name
 * @property string $nit
 * @property string $business_name
 * @property string $time_delivery
 * @property string $created
 * @property string $created_by
 * @property string $updated
 * @property string $updated_by
 */
class Timesdelivery extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'times_delivery';
    }
    
    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'updated'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['updated'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'updated_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['updated_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                Yii::$app->user->identity->username : '',
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['oc_code', 'oc_name', 'nit', 'business_name','time_delivery'], 'required'],
            [['time_delivery', 'nit'], 'integer'],
            [['created', 'updated'], 'safe'],
            [['oc_code'], 'string', 'max' => 45],
            [['oc_name', 'business_name'], 'string', 'max' => 250],
            [['created_by', 'updated_by'], 'string', 'max' => 150],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'oc_code' => Yii::t('app', 'Oc Code'),
            'oc_name' => Yii::t('app', 'Oc Name'),
            'nit' => Yii::t('app', 'Nit'),
            'business_name' => Yii::t('app', 'Business Name'),
            'time_delivery' => Yii::t('app', 'Time Delivery'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'updated' => Yii::t('app', 'Updated'),
            'updated_by' => Yii::t('app', 'Updated By'),
        ];
    }
    
    /**
     * Retorna el listado de los centros de operacion
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public static function getListOC(){
        $clientSiesa = new ClientSiesa;
        $listOC = $clientSiesa->getListOC();
        $data = [];
        
        if(count($listOC)>0){ 
            for ($i = 0; $i < sizeof($listOC); $i++) {
                $data[$listOC[$i]['oc_code']] = $listOC[$i]['oc_name'];
            }               
        } 
        return $data;
    }
    
    /**
     * Retorna el listados de proveedores para el dropdown dle formulario 
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public static function selectProviders($strProvider) {
        $clientSiesa = new ClientSiesa;
        $providers = $clientSiesa->getProviders($strProvider);
        $data = [];
        if(count($providers)>0){                   
            foreach ($providers as $provider) {
                $data[] = ['id' => $provider['nit'], 'text' => $provider['business_name']];  
            }                 
        } 
        return $data;
    }
}
